import json
from typing import Any

from mcp.server.fastmcp import Context, FastMCP
from pydantic import BaseModel

from braze_mcp.models.errors import (
    function_not_found_error,
    internal_error,
    invalid_params_error,
)
from braze_mcp.registry_builder import FUNCTION_REGISTRY
from braze_mcp.utils.context import braze_lifespan

# Defined here to avoid circular dependencies with the tools
mcp = FastMCP(
    "Braze MCP Server",
    lifespan=braze_lifespan,
)

# Cache available function names for better performance
AVAILABLE_FUNCTION_NAMES = list(FUNCTION_REGISTRY.keys())


@mcp.tool()
async def list_functions() -> dict[str, Any]:
    """Lists all available Braze API functions with their descriptions and parameters.

    Returns:
        Dictionary containing all available functions and their metadata
    """
    try:
        available_functions = {}
        for func_name, func_info in FUNCTION_REGISTRY.items():
            available_functions[func_name] = {
                "description": func_info["description"],
                "parameters": func_info["parameters"],
                "returns": func_info["returns"],
            }

        return {
            "available_functions": available_functions,
            "total_functions": len(available_functions),
        }
    except Exception:
        return internal_error("Error listing functions", "list_functions")


@mcp.tool()
async def call_function(
    ctx: Context,
    function_name: str,
    parameters: dict[str, Any] | str | None = None,
) -> Any:
    """Call a specific Braze API function with the provided parameters.

    Args:
        ctx: The MCP context
        function_name: Name of the function to call (use list_functions to see available options)
        parameters: Dictionary of parameters to pass to the function, or JSON string that will be parsed to dictionary (optional)

    Returns:
        The function result as a dictionary or error dictionary
    """
    try:
        if function_name not in FUNCTION_REGISTRY:
            return function_not_found_error(function_name, AVAILABLE_FUNCTION_NAMES)

        func_info = FUNCTION_REGISTRY[function_name]
        implementation = func_info["implementation"]

        # Handle both dict and string parameters
        parsed_parameters = {}
        if parameters is not None:
            if isinstance(parameters, str):
                try:
                    parsed_parameters = json.loads(parameters)
                    if not isinstance(parsed_parameters, dict):
                        return invalid_params_error(
                            "Parameters string must parse to a JSON object/dictionary",
                            "call_function",
                        )
                except json.JSONDecodeError:
                    return invalid_params_error(
                        "Invalid JSON in parameters string", "call_function"
                    )
            elif isinstance(parameters, dict):
                parsed_parameters = parameters
            else:
                return invalid_params_error(
                    f"Parameters must be a dictionary or JSON string, got {type(parameters).__name__}",
                    "call_function",
                )

        # Call the function with context as first parameter
        result = await implementation(ctx, **parsed_parameters)

        # Convert Pydantic models to dictionaries for MCP transport with schema context
        if isinstance(result, BaseModel):
            return {
                "data": result.model_dump(),
                "schema": {
                    "model_name": result.__class__.__name__,
                    "fields": result.model_json_schema(),
                    "description": f"Response data structured according to the {result.__class__.__name__} model",
                },
            }

        return result

    except Exception:
        return internal_error(f"Error calling function '{function_name}'", "call_function")



from unittest.mock import MagicMock, patch

import pytest

from braze_mcp.models.events import EventListResponse
from braze_mcp.tools.events import get_events_list


class TestGetEventsList:
    """Test get_events_list function"""

    @pytest.mark.asyncio
    async def test_get_events_list_success_with_default_parameters(
        self, mock_context, mock_braze_context, sample_events_list_data
    ):
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.events.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.events.make_request",
                return_value=SuccessResponse(data=sample_events_list_data, headers={}),
            ) as mock_request:
                result = await get_events_list(mock_context)

                assert isinstance(result, EventListResponse)
                assert result.message == "success"
                assert len(result.events) == 3
                assert result.events[0] == "Event A"
                assert result.events[1] == "Event B"
                assert result.events[2] == "Event C"

                # Verify default parameters
                call_args = mock_request.call_args
                assert call_args[0][0] == mock_braze_context.http_client  # client
                assert call_args[0][1] == mock_braze_context.base_url  # base_url
                assert call_args[0][2] == "events/list"  # url_path
                params = call_args[0][3]  # params
                assert params["page"] == 0

    @pytest.mark.asyncio
    async def test_get_events_list_success_with_custom_page(
        self, mock_context, mock_braze_context, sample_events_list_data
    ):
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.events.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.events.make_request",
                return_value=SuccessResponse(data=sample_events_list_data, headers={}),
            ) as mock_request:
                await get_events_list(mock_context, page=3)

                # Verify parameters were passed correctly
                call_args = mock_request.call_args
                params = call_args[0][3]  # params
                assert params["page"] == 3

    @pytest.mark.asyncio
    async def test_get_events_list_request_failure(self, mock_context, mock_braze_context):
        from braze_mcp.utils.http import FailureResponse

        with patch(
            "braze_mcp.tools.events.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.events.make_request",
                return_value=FailureResponse(
                    data={"error": "Request failed"},
                    error=Exception("Request failed"),
                ),
            ):
                result = await get_events_list(mock_context)

                assert isinstance(result, dict)
                assert "error" in result
                assert result["error"] == "Request failed"

    @pytest.mark.asyncio
    async def test_get_events_list_response_fails_parsing(self, mock_context, mock_braze_context):
        """Test get events list returns the raw response when the response fails schema validation"""
        invalid_data = {"message": "success", "invalid_field": "test"}
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.events.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.events.make_request",
                return_value=SuccessResponse(data=invalid_data, headers={}),
            ):
                with patch("braze_mcp.tools.events.logger.exception") as mock_logger:
                    result = await get_events_list(mock_context)

                    assert isinstance(result, dict)
                    assert result == invalid_data
                    mock_logger.assert_called_once()

    @pytest.mark.asyncio
    async def test_get_events_list_unexpected_response_type(self, mock_context, mock_braze_context):
        """Test events list handles unexpected response type"""
        unexpected_response = MagicMock()
        unexpected_response.__class__.__name__ = "UnexpectedResponse"

        with patch(
            "braze_mcp.tools.events.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.events.make_request",
                return_value=unexpected_response,
            ):
                with patch("braze_mcp.tools.events.logger.error") as mock_logger:
                    result = await get_events_list(mock_context)

                    assert isinstance(result, dict)
                    assert "error" in result
                    error_obj = result["error"]
                    assert error_obj["error_type"] == "unexpected_error"
                    assert "Unexpected response type:" in error_obj["message"]
                    mock_logger.assert_called_once_with(
                        f"Unexpected response type: {type(unexpected_response)}"
                    )



from unittest.mock import patch

import pytest

from braze_mcp.models import SDKAuthenticationKeysResponse
from braze_mcp.tools.sdk_authentication import get_sdk_authentication_keys


@pytest.fixture
def sample_sdk_authentication_keys_data():
    return {
        "keys": [
            {
                "id": "test-key-id",
                "rsa_public_key": "-----BEGIN PUBLIC KEY-----\nTEST_KEY\n-----END PUBLIC KEY-----",
                "description": "Test SDK Key",
                "is_primary": True,
            }
        ]
    }


class TestGetSDKAuthenticationKeys:
    """Test get_sdk_authentication_keys function"""

    @pytest.mark.asyncio
    async def test_success_response(
        self, mock_context, mock_braze_context, sample_sdk_authentication_keys_data
    ):
        """Test successful retrieval of SDK authentication keys"""
        from braze_mcp.utils.http import SuccessResponse

        app_id = "test-app-id"

        with (
            patch(
                "braze_mcp.tools.sdk_authentication.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.sdk_authentication.make_request",
                return_value=SuccessResponse(data=sample_sdk_authentication_keys_data, headers={}),
            ) as mock_request,
        ):
            result = await get_sdk_authentication_keys(mock_context, app_id)

            assert isinstance(result, SDKAuthenticationKeysResponse)
            assert len(result.keys) == 1

            key = result.keys[0]
            assert key.id == "test-key-id"
            assert key.is_primary is True
            assert "BEGIN PUBLIC KEY" in key.rsa_public_key

            # Verify API call
            call_args = mock_request.call_args
            assert call_args[0][2] == "app_group/sdk_authentication/keys"
            assert call_args[0][3]["app_id"] == app_id

    @pytest.mark.asyncio
    async def test_error_response(self, mock_context, mock_braze_context):
        """Test handling of API errors"""
        from braze_mcp.utils.http import FailureResponse

        app_id = "test-app-id"

        with (
            patch(
                "braze_mcp.tools.sdk_authentication.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.sdk_authentication.make_request",
                return_value=FailureResponse(
                    data={"error": "Invalid API key"}, error=Exception("Unauthorized")
                ),
            ),
        ):
            result = await get_sdk_authentication_keys(mock_context, app_id)

            assert isinstance(result, dict)
            assert "error" in result



from typing import Any

from mcp.server.fastmcp import Context

from braze_mcp.models import (
    ProductListResponse,
    QuantitySeriesResponse,
    RevenueSeriesResponse,
)
from braze_mcp.utils import get_braze_context, get_logger, make_request
from braze_mcp.utils.http import (
    handle_response,
)

__register_mcp_tools__ = True

logger = get_logger(__name__)


async def get_product_list(
    ctx: Context, page: str | None = None
) -> ProductListResponse | dict[str, Any]:
    """Use this endpoint to return a paginated lists of product IDs.

    Args:
        ctx: The MCP context
        page: The page of your product list that you want to view

    Returns:
        ProductListResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    url_path = "purchases/product_list"

    params = {
        "page": page,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, ProductListResponse, "fetch product list", logger)


async def get_revenue_series(
    ctx: Context,
    length: int,
    ending_at: str | None = None,
    unit: str | None = None,
    app_id: str | None = None,
    product: str | None = None,
) -> RevenueSeriesResponse | dict[str, Any]:
    """Use this endpoint to return the total money spent in your app over a time range.

    Args:
        ctx: The MCP context
        length: Maximum number of days before ending_at to include in the returned series. Must be between 1 and 100 (inclusive)
        ending_at: Date on which the data export should end (ISO-8601 string). Defaults to time of the request
        unit: Unit of time between data points. Can be day or hour, defaults to day
        app_id: App API identifier retrieved from the API Keys page. If excluded, results for all apps in a workspace will be returned
        product: Name of product to filter response by. If excluded, results for all apps will be returned

    Returns:
        RevenueSeriesResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    url_path = "purchases/revenue_series"

    params = {
        "length": length,
        "ending_at": ending_at,
        "unit": unit,
        "app_id": app_id,
        "product": product,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, RevenueSeriesResponse, "fetch revenue series", logger)


async def get_quantity_series(
    ctx: Context,
    length: int,
    ending_at: str | None = None,
    unit: str | None = None,
    app_id: str | None = None,
    product: str | None = None,
) -> QuantitySeriesResponse | dict[str, Any]:
    """Use this endpoint to return the total number of purchases in your app over a time range.

    Args:
        ctx: The MCP context
        length: Maximum number of days before ending_at to include in the returned series. Must be between 1 and 100 (inclusive)
        ending_at: Date on which the data export should end (ISO-8601 string). Defaults to time of the request
        unit: Unit of time between data points. Can be day or hour, defaults to day
        app_id: App API identifier retrieved from the API Keys page. If excluded, results for all apps in a workspace will be returned
        product: Name of product to filter response by. If excluded, results for all apps will be returned

    Returns:
        QuantitySeriesResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    url_path = "purchases/quantity_series"

    params = {
        "length": length,
        "ending_at": ending_at,
        "unit": unit,
        "app_id": app_id,
        "product": product,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, QuantitySeriesResponse, "fetch quantity series", logger)



from unittest.mock import patch

import pytest

from braze_mcp.registry_builder import (
    DOCSTRING_SECTION_HEADERS,
    FUNCTION_REGISTRY,
    _extract_description_from_docstring,
    _extract_param_description,
    _is_section_header,
    _log_docstring_validation_warnings,
    _parse_args_section,
    _parse_returns_section,
    _python_type_to_json_type,
    _safe_serialize_default,
    build_function_registry,
    extract_function_metadata,
    validate_docstring,
)


class TestFunctionRegistryBuilder:
    """Test the function registry auto-discovery system"""

    def test_get_campaign_list_metadata_extraction(self):
        """Test detailed metadata extraction for get_campaign_list function"""
        registry = build_function_registry()
        campaign_list_func = registry["get_campaign_list"]

        # Test function description
        assert "export a list of campaigns" in campaign_list_func["description"]

        # Test parameters structure
        params = campaign_list_func["parameters"]

        # Should have all expected parameters (including ctx)
        expected_params = {
            "ctx",
            "page",
            "include_archived",
            "sort_direction",
            "last_edit_time_gt",
        }
        assert set(params.keys()) == expected_params

        # Test page parameter
        page_param = params["page"]
        assert page_param["type"] == "integer"
        assert page_param["required"] is False
        assert page_param["default"] == 0
        assert "page" in page_param["description"].lower()

        # Test include_archived parameter
        archived_param = params["include_archived"]
        assert archived_param["type"] == "boolean"
        assert archived_param["required"] is False
        assert archived_param["default"] is False
        assert "archived" in archived_param["description"].lower()

        # Test sort_direction parameter (str with default)
        sort_param = params["sort_direction"]
        assert sort_param["type"] == "string"
        assert sort_param["required"] is False
        assert sort_param["default"] == "desc"
        assert "sort" in sort_param["description"].lower()

        # Test last_edit_time_gt parameter (Optional[str])
        time_param = params["last_edit_time_gt"]
        assert time_param["type"] == "string"
        assert time_param["required"] is False
        assert time_param["default"] is None
        assert "time" in time_param["description"].lower()

    def test_get_campaign_details_required_parameter(self):
        """Test that required parameters are correctly identified"""
        registry = build_function_registry()
        campaign_details_func = registry["get_campaign_details"]

        params = campaign_details_func["parameters"]

        # campaign_id should be required
        campaign_id_param = params["campaign_id"]
        assert campaign_id_param["required"] is True
        assert campaign_id_param["type"] == "string"
        assert "default" not in campaign_id_param  # Required params shouldn't have defaults

        # post_launch_draft_version should be optional
        draft_param = params["post_launch_draft_version"]
        assert draft_param["required"] is False
        assert draft_param["type"] == "boolean"
        assert draft_param["default"] is False


class TestMetadataExtractionHelpers:
    """Test the helper functions for metadata extraction"""

    def test_python_type_to_json_type_conversion(self):
        """Test type conversion from Python types to JSON schema types"""
        assert _python_type_to_json_type(str) == "string"
        assert _python_type_to_json_type(int) == "integer"
        assert _python_type_to_json_type(bool) == "boolean"
        assert _python_type_to_json_type(float) == "number"
        assert _python_type_to_json_type(list) == "array"
        assert _python_type_to_json_type(dict) == "object"

        # Test unknown type defaults to object
        class CustomType:
            pass

        assert _python_type_to_json_type(CustomType) == "object"

    def test_extract_description_from_docstring(self):
        """Test extracting descriptions from function docstrings"""
        # Test with proper docstring - should extract multiline description
        docstring = """Export a list of campaigns with their names.

        This function retrieves campaign data from the Braze API.

        Args:
            page: Page number
        """
        expected = "Export a list of campaigns with their names. This function retrieves campaign data from the Braze API."
        assert _extract_description_from_docstring(docstring) == expected

        # Test with None docstring
        assert _extract_description_from_docstring(None) == "No description available"

        # Test with empty docstring
        assert _extract_description_from_docstring("") == "No description available"

        # Test with whitespace-only docstring
        assert _extract_description_from_docstring("   \n   \n   ") == "No description available"

    def test_extract_param_description_from_docstring(self):
        """Test extracting parameter descriptions from docstrings (no fallback)"""
        docstring = """Function description.

        Args:
            page: Page number for pagination
            campaign_id: The unique campaign identifier
            include_archived: Whether to include archived items
        """

        # Test extracting existing parameter descriptions
        assert "pagination" in _extract_param_description(docstring, "page", int)
        assert "unique campaign identifier" in _extract_param_description(
            docstring, "campaign_id", str
        )
        assert "archived items" in _extract_param_description(docstring, "include_archived", bool)

        # Test parameter not in docstring - no fallback, just basic description
        result = _extract_param_description(docstring, "unknown_param", str)
        assert result == "Parameter unknown_param"

        # Test with no docstring
        result_no_doc = _extract_param_description(None, "some_param", str)
        assert result_no_doc == "Parameter some_param"

    def test_parse_args_section_direct(self):
        """Test the Args section parser directly"""
        docstring = """Function description.

        Args:
            simple_param: Simple description
            multi_line_param: This is a parameter with a very long description
                that spans multiple lines and should be properly joined
            type_param (str): Parameter with type annotation
            complex_param: First line
                Second line of description

        Returns:
            Something useful
        """

        # Test simple parameter
        result = _parse_args_section(docstring, "simple_param")
        assert result == "Simple description"

        # Test multi-line parameter
        result = _parse_args_section(docstring, "multi_line_param")
        assert "very long description that spans multiple lines" in result

        # Test parameter with type annotation
        result = _parse_args_section(docstring, "type_param")
        assert "Parameter with type annotation" in result

        # Test complex multi-line
        result = _parse_args_section(docstring, "complex_param")
        assert "First line Second line of description" in result

        # Test parameter not found
        result = _parse_args_section(docstring, "missing_param")
        assert result is None

        # Test with no Args section
        no_args_docstring = """Just a description.

        Returns:
            Something
        """
        result = _parse_args_section(no_args_docstring, "any_param")
        assert result is None

    def test_parse_returns_section(self):
        """Test the Returns section parser"""
        # Test with Returns section
        docstring_returns = """Function description.

        Args:
            param: Some parameter

        Returns:
            JSON string containing the response data
        """
        result = _parse_returns_section(docstring_returns)
        assert result == "JSON string containing the response data"

        # Test with multi-line Returns
        docstring_multiline = """Function description.

        Returns:
            A complex response object containing multiple fields
            and detailed information about the operation
        """
        result = _parse_returns_section(docstring_multiline)
        assert "complex response object" in result
        assert "detailed information" in result

        # Test with Return (singular)
        docstring_return = """Function description.

        Return:
            Simple return value
        """
        result = _parse_returns_section(docstring_return)
        assert result == "Simple return value"

        # Test with no Returns section
        docstring_no_returns = """Function description.

        Args:
            param: Some parameter
        """
        result = _parse_returns_section(docstring_no_returns)
        assert result is None

        # Test with None docstring
        result = _parse_returns_section(None)
        assert result is None

    def test_function_registry_includes_returns_metadata(self):
        """Test that function registry includes returns information"""
        registry = build_function_registry()

        # Check that functions have returns metadata
        for func_name in [
            "get_campaign_list",
            "get_campaign_details",
            "get_custom_attributes",
        ]:
            func_info = registry[func_name]

            # Should have returns information
            assert "returns" in func_info
            returns_info = func_info["returns"]

            # Returns should have description and type
            assert "description" in returns_info
            assert "type" in returns_info
            assert returns_info["type"] == "object"

            # Description should contain meaningful content about structured data
            description_lower = returns_info["description"].lower()
            assert any(
                keyword in description_lower
                for keyword in ["dict", "dictionary", "data", "containing"]
            )
            assert len(returns_info["description"]) > 10  # Should be descriptive

    def test_optional_type_handling(self):
        """Test that modern union types are handled correctly"""
        # Test modern optional syntax
        result = _python_type_to_json_type(str | None)
        assert result == "string"

        # Test modern union syntax
        result = _python_type_to_json_type(str | int)
        assert result == "string"

        # Test modern int | None syntax
        result = _python_type_to_json_type(int | None)
        assert result == "integer"

    def test_registry_is_built_on_import(self):
        """Test that FUNCTION_REGISTRY is automatically built on import"""
        # The registry should be populated when the module is imported
        assert isinstance(FUNCTION_REGISTRY, dict)
        assert len(FUNCTION_REGISTRY) > 0

        # Should contain our expected functions
        expected_functions = {
            "get_campaign_list",
            "get_campaign_details",
            "get_custom_attributes",
        }
        assert expected_functions.issubset(set(FUNCTION_REGISTRY.keys()))


class TestEnhancedTypeConversion:
    """Test enhanced type conversion functionality"""

    def test_context_type_handling(self):
        """Test that Context type is properly converted to object"""
        from mcp.server.fastmcp import Context

        result = _python_type_to_json_type(Context)
        assert result == "object"

    def test_generic_list_types(self):
        """Test List and generic array types"""
        result = _python_type_to_json_type(list[str])
        assert result == "array"

        result = _python_type_to_json_type(list)
        assert result == "array"

    def test_generic_dict_types(self):
        """Test Dict and generic object types"""
        result = _python_type_to_json_type(dict[str, str])
        assert result == "object"

        result = _python_type_to_json_type(dict)
        assert result == "object"

    def test_complex_union_types(self):
        """Test complex union type handling with modern syntax"""
        # Test modern str | int syntax
        result = _python_type_to_json_type(str | int)
        assert result == "string"

        # Test modern int | str | None syntax
        result = _python_type_to_json_type(int | str | None)
        assert result == "integer"

    def test_response_model_types(self):
        """Test that Response/Request/Model classes are treated as objects"""

        class TestResponse:
            pass

        class CustomRequest:
            pass

        class SomeModel:
            pass

        assert _python_type_to_json_type(TestResponse) == "object"
        assert _python_type_to_json_type(CustomRequest) == "object"
        assert _python_type_to_json_type(SomeModel) == "object"

    def test_type_conversion_error_handling(self):
        """Test that type conversion handles errors gracefully"""

        # Create a problematic type that might cause errors
        class ProblematicType:
            def __getattr__(self, name):
                raise AttributeError("Simulated error")

        result = _python_type_to_json_type(ProblematicType)
        assert result == "object"  # Should fall back to object


class TestSafeDefaultSerialization:
    """Test safe default value serialization"""

    def test_json_serializable_defaults(self):
        """Test that JSON-serializable defaults pass through unchanged"""
        assert _safe_serialize_default(42) == 42
        assert _safe_serialize_default("hello") == "hello"
        assert _safe_serialize_default(True) is True
        assert _safe_serialize_default(None) is None
        assert _safe_serialize_default([1, 2, 3]) == [1, 2, 3]
        assert _safe_serialize_default({"key": "value"}) == {"key": "value"}

    def test_function_defaults(self):
        """Test that function defaults are converted to strings"""

        def sample_func():
            pass

        result = _safe_serialize_default(sample_func)
        assert result == "<function: sample_func>"

    def test_object_defaults(self):
        """Test that complex object defaults are converted to strings"""

        class TestClass:
            def __str__(self):
                return "test_instance"

        obj = TestClass()
        result = _safe_serialize_default(obj)
        assert result == "<TestClass: test_instance>"

    def test_non_serializable_defaults(self):
        """Test handling of various non-serializable types"""
        import datetime

        # Test datetime object
        dt = datetime.datetime.now()
        result = _safe_serialize_default(dt)
        assert isinstance(result, str)
        assert "datetime" in result.lower()


class TestRobustParameterMatching:
    """Test enhanced parameter matching in docstrings"""

    def test_exact_parameter_matching(self):
        """Test that parameter matching is exact and doesn't match substrings"""
        docstring = """Function description.

        Args:
            sort_direction: Main sort parameter
            sort_direction_other: Another sort parameter
            direction: Simple direction
        """

        # Should match exact parameter name only
        result = _parse_args_section(docstring, "sort_direction")
        assert result == "Main sort parameter"

        # Should not match partial names
        result = _parse_args_section(docstring, "sort")
        assert result is None

    def test_parameter_with_type_annotations(self):
        """Test parameter parsing with type annotations in docstring"""
        docstring = """Function description.

        Args:
            param_name (str): Description with type annotation
            another_param (Optional[int]): Another parameter with complex type
            simple_param: Parameter without type annotation
        """

        result = _parse_args_section(docstring, "param_name")
        assert result == "Description with type annotation"

        result = _parse_args_section(docstring, "another_param")
        assert result == "Another parameter with complex type"

        result = _parse_args_section(docstring, "simple_param")
        assert result == "Parameter without type annotation"

    def test_dynamic_indentation_handling(self):
        """Test that indentation detection works with various formatting"""
        docstring = """Function description.

        Args:
            param1: Description that continues
                on multiple lines with proper
                indentation handling
            param2: Another parameter
        """

        result = _parse_args_section(docstring, "param1")
        expected = "Description that continues on multiple lines with proper indentation handling"
        assert result == expected


class TestErrorHandling:
    """Test enhanced error handling throughout the pipeline"""

    def test_extract_metadata_with_type_hint_errors(self):
        """Test that type hint errors don't crash metadata extraction"""

        # Create a mock function with problematic type hints
        def problematic_func(param: "NonExistentType") -> str:  # noqa: F821  # Forward ref that doesn't exist
            """Test function with bad type hints.

            Args:
                param: A parameter with problematic type

            Returns:
                A string
            """
            return "test"

        with patch("braze_mcp.registry_builder.logger.warning") as mock_warning:
            metadata = extract_function_metadata(problematic_func)

            # Should have warning about type hints but still extract metadata
            mock_warning.assert_called()
            assert any(
                "Could not get type hints" in str(call) for call in mock_warning.call_args_list
            )

            # Should still have basic metadata
            assert metadata["description"] == "Test function with bad type hints."
            assert "param" in metadata["parameters"]
            assert metadata["parameters"]["param"]["type"] == "string"  # fallback

    def test_extract_metadata_with_parameter_errors(self):
        """Test that individual parameter errors don't break entire extraction"""

        def func_with_issues(normal_param: str, problematic_param) -> str:
            """Function with mixed parameter issues.

            Args:
                normal_param: This is fine
                # Missing problematic_param description

            Returns:
                A string
            """
            return "test"

        metadata = extract_function_metadata(func_with_issues)

        # Should have metadata for both parameters
        assert "normal_param" in metadata["parameters"]
        assert "problematic_param" in metadata["parameters"]

        # Normal param should have proper description
        assert "This is fine" in metadata["parameters"]["normal_param"]["description"]

        # Problematic param should have fallback description
        assert (
            "Parameter problematic_param"
            == metadata["parameters"]["problematic_param"]["description"]
        )

    def test_extract_metadata_complete_failure(self):
        """Test that complete metadata extraction failures are handled"""

        # Create an object that looks like a function but will cause errors
        class NotAFunction:
            __name__ = "fake_function"

        fake_func = NotAFunction()

        with patch("builtins.print") as mock_print:
            metadata = extract_function_metadata(fake_func)

            # Should have error message
            error_calls = [call for call in mock_print.call_args_list if "Error" in str(call)]
            assert len(error_calls) > 0

            # Should return minimal metadata with error info
            assert metadata["description"] == "Function fake_function (metadata extraction failed)"
            assert metadata["parameters"] == {}
            assert "error" in metadata


class TestMultilineDescriptions:
    """Test the enhanced multiline description extraction"""

    def test_multiline_description_extraction(self):
        """Test that multiline descriptions are properly extracted"""
        docstring = """Create a new campaign in Braze.

        This function allows you to create campaigns with various settings
        and configurations for targeted messaging. It provides comprehensive
        control over campaign parameters.

        Args:
            param: Some parameter
        """

        result = _extract_description_from_docstring(docstring)
        expected = (
            "Create a new campaign in Braze. This function allows you to create "
            "campaigns with various settings and configurations for targeted messaging. "
            "It provides comprehensive control over campaign parameters."
        )
        assert result == expected

    def test_description_stops_at_sections(self):
        """Test that description extraction stops at section headers"""
        docstring = """Main description line.

        Continued description here.

        Args:
            This should not be in description
        """

        result = _extract_description_from_docstring(docstring)
        assert result == "Main description line. Continued description here."
        assert "Args:" not in result
        assert "This should not be in description" not in result

    def test_description_handles_various_sections(self):
        """Test that description stops at various section types"""
        sections = ["Args:", "Returns:", "Raises:", "Examples:", "Note:", "Warning:"]

        for section in sections:
            docstring = f"""Main description.

            More description content.

            {section}
                Section content here
            """

            result = _extract_description_from_docstring(docstring)
            assert result == "Main description. More description content."
            assert section not in result


class TestDocstringValidation:
    """Test the docstring validation system"""

    def test_valid_docstring(self):
        """Test that a properly formatted docstring passes validation"""

        def good_function(ctx, param1: str, param2: int = 0) -> dict:
            """Brief description of the function.

            Args:
                param1: Description of param1
                param2: Description of param2 with default

            Returns:
                Dictionary containing result data
            """
            pass

        is_valid, issues = validate_docstring(good_function)
        assert is_valid
        assert len(issues) == 0

    def test_missing_docstring(self):
        """Test that missing docstring is caught"""

        def bad_function(ctx, param1: str) -> dict:
            pass

        is_valid, issues = validate_docstring(bad_function)
        assert not is_valid
        assert "Missing docstring" in issues

    def test_missing_args_section(self):
        """Test that missing Args section is caught"""

        def bad_function(ctx, param1: str, param2: int) -> dict:
            """Function description without Args section."""
            pass

        is_valid, issues = validate_docstring(bad_function)
        assert not is_valid
        assert any("Args" in issue for issue in issues)

    def test_undocumented_parameter(self):
        """Test that undocumented parameters are caught"""

        def bad_function(ctx, param1: str, param2: int) -> dict:
            """Function description.

            Args:
                param1: Documented parameter
                # param2 is missing!
            """
            pass

        is_valid, issues = validate_docstring(bad_function)
        assert not is_valid
        assert any("param2" in issue for issue in issues)

    def test_ctx_parameter_ignored(self):
        """Test that ctx parameter doesn't need documentation"""

        def bad_function(ctx) -> dict:
            """Function that only takes ctx parameter."""
            pass

        is_valid, issues = validate_docstring(bad_function)
        assert not is_valid  # Returns section is still required
        assert "Missing Returns section" in issues

        def good_function(ctx) -> dict:
            """Function that only takes ctx parameter.

            Returns:
                Dictionary containing result data
            """
            pass

        is_valid, issues = validate_docstring(good_function)
        assert is_valid  # ctx-only functions don't need Args section but do need Returns

    def test_missing_returns_section(self):
        """Test that missing Returns section is caught"""

        def bad_function(ctx, param1: str) -> dict:
            """Function description.

            Args:
                param1: Documented parameter
            """
            pass

        is_valid, issues = validate_docstring(bad_function)
        assert not is_valid
        assert "Missing Returns section" in issues

    def test_empty_returns_section(self):
        """Test that empty Returns section is caught"""

        def bad_function(ctx, param1: str) -> dict:
            """Function description.

            Args:
                param1: Documented parameter

            Returns:
            """
            pass

        is_valid, issues = validate_docstring(bad_function)
        assert not is_valid
        assert "Returns section must contain a description" in issues

    def test_validation_logging(self, caplog):
        """Test that validation warnings are logged properly"""

        def bad_function(ctx, param1: str) -> dict:
            """Bad function."""
            pass

        is_valid, issues = validate_docstring(bad_function)
        _log_docstring_validation_warnings("test_function", issues)

        assert "Docstring issues in function 'test_function':" in caplog.text
        assert "Consider following the docstring conventions" in caplog.text

    def test_strict_validation_in_registry_building(self):
        """Test that functions with invalid docstrings fail to register in strict mode"""
        from types import ModuleType

        # Create a temporary module
        temp_module = ModuleType("temp_tools")
        temp_module.__name__ = "braze_mcp.tools.temp"
        temp_module.__register_mcp_tools__ = True

        # Add a function with invalid docstring
        async def bad_tool_function(ctx, param1: str) -> dict:
            """Function without Args section."""
            return {}

        # Set the module attribute so it looks like it belongs to tools
        bad_tool_function.__module__ = "braze_mcp.tools.temp"
        temp_module.get_bad_tool = bad_tool_function

        # Patch the discovery function to include our bad module
        with patch(
            "braze_mcp.registry_builder._discover_mcp_tool_modules",
            return_value=[temp_module],
        ):
            # This should raise an error due to strict validation
            with pytest.raises(ValueError, match="has invalid docstring and cannot be registered"):
                build_function_registry()


class TestSectionHeaderDetection:
    """Test the _is_section_header helper function"""

    def test_basic_section_header_detection(self):
        """Test basic section header detection without specific sections"""
        # Valid section headers
        assert _is_section_header("Args:")
        assert _is_section_header("Returns:")
        assert _is_section_header("Notes:")
        assert _is_section_header("Custom Section:")

        # Invalid - indented
        assert not _is_section_header("    Args:")
        assert not _is_section_header("\tReturns:")

        # Invalid - no colon
        assert not _is_section_header("Args")
        assert not _is_section_header("Returns")

        # Invalid - empty or whitespace
        assert not _is_section_header("")
        assert not _is_section_header("   ")

    def test_specific_section_validation(self):
        """Test section header detection with specific valid sections"""
        valid_sections = ["args:", "returns:", "notes:"]

        # Valid sections
        assert _is_section_header("Args:", valid_sections)
        assert _is_section_header("ARGS:", valid_sections)  # Case insensitive
        assert _is_section_header("returns:", valid_sections)
        assert _is_section_header("Notes:", valid_sections)

        # Invalid sections (not in list)
        assert not _is_section_header("Examples:", valid_sections)
        assert not _is_section_header("Custom:", valid_sections)

        # Still invalid for basic formatting reasons
        assert not _is_section_header("    args:", valid_sections)
        assert not _is_section_header("args", valid_sections)

    def test_docstring_section_headers_validation(self):
        """Test with the actual DOCSTRING_SECTION_HEADERS list"""
        # Valid headers from the constant
        assert _is_section_header("Args:", DOCSTRING_SECTION_HEADERS)
        assert _is_section_header("Arguments:", DOCSTRING_SECTION_HEADERS)
        assert _is_section_header("Parameters:", DOCSTRING_SECTION_HEADERS)
        assert _is_section_header("Returns:", DOCSTRING_SECTION_HEADERS)
        assert _is_section_header("Return:", DOCSTRING_SECTION_HEADERS)
        assert _is_section_header("Raises:", DOCSTRING_SECTION_HEADERS)
        assert _is_section_header("Examples:", DOCSTRING_SECTION_HEADERS)

        # Case insensitive
        assert _is_section_header("ARGS:", DOCSTRING_SECTION_HEADERS)
        assert _is_section_header("returns:", DOCSTRING_SECTION_HEADERS)

        # Invalid header
        assert not _is_section_header("Custom:", DOCSTRING_SECTION_HEADERS)

    def test_real_docstring_scenarios(self):
        """Test with realistic docstring line scenarios"""
        # Typical docstring lines that should be detected as headers
        assert _is_section_header("Args:")
        assert _is_section_header("Returns:")

        # Should NOT be detected as headers (indented)
        assert not _is_section_header("    Returns:")
        assert not _is_section_header("\tArgs:")

        # Parameter descriptions (should not be headers)
        assert not _is_section_header("    param1: Description of parameter")
        assert not _is_section_header("        campaign_id: The campaign identifier")

        # Regular text (should not be headers)
        assert not _is_section_header("This is a regular description line.")
        assert not _is_section_header("Some text with a colon: but indented")



from typing import Any

from mcp.server.fastmcp import Context

from braze_mcp.models import SendDataSeriesResponse
from braze_mcp.utils import get_braze_context, get_logger, make_request
from braze_mcp.utils.http import (
    handle_response,
)

__register_mcp_tools__ = True

logger = get_logger(__name__)


async def get_send_data_series(
    ctx: Context,
    campaign_id: str,
    send_id: str,
    length: int,
    ending_at: str | None = None,
) -> SendDataSeriesResponse | dict[str, Any]:
    """Use this endpoint to retrieve a daily series of various stats for a tracked send_id for API campaigns.

    Braze stores send analytics for 14 days after the send. Campaign conversions will be attributed toward
    the most recent send_id that a given user has received from the campaign.

    Args:
        ctx: The MCP context
        campaign_id: The campaign_id for API campaigns can be found on the API Keys page and the Campaign Details page within your dashboard; or you can use the Export campaigns list endpoint.
        send_id: The Send API identifier for the specific send
        length: Maximum number of days before ending_at to include in the returned series. Must be between 1 and 100 (inclusive).
        ending_at: Date on which the data series should end (format: ISO-8601 datetime string). Defaults to time of the request.

    Returns:
        SendDataSeriesResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    url_path = "sends/data_series"

    params = {
        "campaign_id": campaign_id,
        "send_id": send_id,
        "length": length,
        "ending_at": ending_at,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, SendDataSeriesResponse, "fetch send data series", logger)



from pydantic import Field

from .common import BrazeBaseModel


class IOSPushMessageStatistics(BrazeBaseModel):
    """
    Model representing iOS push message statistics.
    """

    variation_name: str | None = Field(None, description="The variation name")
    variation_api_id: str | None = Field(None, description="The variation API identifier")
    sent: int | None = Field(None, description="The number of sends")
    delivered: int | None = Field(None, description="The number of messages successfully delivered")
    undelivered: int | None = Field(None, description="The number of undelivered")
    delivery_failed: int | None = Field(None, description="The number of rejected")
    direct_opens: int | None = Field(None, description="The number of direct opens")
    total_opens: int | None = Field(None, description="The number of total opens")
    bounces: int | None = Field(None, description="The number of bounces")
    body_clicks: int | None = Field(None, description="The number of body clicks")
    revenue: float | None = Field(None, description="The number of dollars of revenue (USD)")
    unique_recipients: int | None = Field(None, description="The number of unique recipients")
    conversions: int | None = Field(None, description="The number of conversions")
    conversions_by_send_time: int | None = Field(
        None, description="The number of conversions attributed to the date the campaign was sent"
    )
    conversions1: int | None = Field(
        None, description="The number of conversions for the second conversion event"
    )
    conversions1_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the second conversion event attributed to the date the campaign was sent",
    )
    conversions2: int | None = Field(
        None, description="The number of conversions for the third conversion event"
    )
    conversions2_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the third conversion event attributed to the date the campaign was sent",
    )
    conversions3: int | None = Field(
        None, description="The number of conversions for the fourth conversion event"
    )
    conversions3_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the fourth conversion event attributed to the date the campaign was sent",
    )
    enrolled: int | None = Field(None, description="The number of enrolled users")


class AndroidPushMessageStatistics(BrazeBaseModel):
    """
    Model representing Android push message statistics.
    """

    variation_api_id: str = Field(..., description="The variation API identifier")
    sent: int = Field(..., description="The number of sends")
    direct_opens: int = Field(..., description="The number of direct opens")
    total_opens: int = Field(..., description="The number of total opens")
    bounces: int = Field(..., description="The number of bounces")
    body_clicks: int = Field(..., description="The number of body clicks")


class WebhookMessageStatistics(BrazeBaseModel):
    """
    Model representing webhook message statistics.
    """

    variation_name: str | None = Field(None, description="The variation name")
    variation_api_id: str = Field(..., description="The variation API identifier")
    sent: int | None = Field(None, description="The number of sends")
    errors: int | None = Field(None, description="The number of errors")
    revenue: float | None = Field(None, description="The number of dollars of revenue (USD)")
    unique_recipients: int | None = Field(None, description="The number of unique recipients")
    conversions: int | None = Field(None, description="The number of conversions")
    conversions_by_send_time: int | None = Field(
        None, description="The number of conversions attributed to the date the campaign was sent"
    )
    conversions1: int | None = Field(
        None, description="The number of conversions for the second conversion event"
    )
    conversions1_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the second conversion event attributed to the date the campaign was sent",
    )
    conversions2: int | None = Field(
        None, description="The number of conversions for the third conversion event"
    )
    conversions2_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the third conversion event attributed to the date the campaign was sent",
    )
    conversions3: int | None = Field(
        None, description="The number of conversions for the fourth conversion event"
    )
    conversions3_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the fourth conversion event attributed to the date the campaign was sent",
    )
    enrolled: int | None = Field(None, description="The number of enrolled users")


class EmailMessageStatistics(BrazeBaseModel):
    """
    Model representing email message statistics.
    """

    variation_name: str | None = Field(None, description="The variation name")
    variation_api_id: str | None = Field(None, description="The variation API identifier")
    sent: int | None = Field(None, description="The number of sends")
    opens: int | None = Field(None, description="The number of opens")
    unique_opens: int | None = Field(None, description="The number of unique opens")
    clicks: int | None = Field(None, description="The number of clicks")
    unique_clicks: int | None = Field(None, description="The number of unique clicks")
    unsubscribes: int | None = Field(None, description="The number of unsubscribes")
    bounces: int | None = Field(None, description="The number of bounces")
    delivered: int | None = Field(None, description="The number of messages delivered")
    reported_spam: int | None = Field(None, description="The number of messages reported as spam")


class SMSMessageStatistics(BrazeBaseModel):
    """
    Model representing SMS message statistics.
    """

    variation_api_id: str = Field(..., description="The variation API identifier")
    sent: int = Field(..., description="The number of sends")
    sent_to_carrier: int = Field(..., description="The number of messages sent to the carrier")
    delivered: int = Field(..., description="The number of delivered messages")
    rejected: int = Field(..., description="The number of rejected messages")
    delivery_failed: int = Field(..., description="The number of failed deliveries")
    clicks: int = Field(..., description="The number of clicks on shortened links")
    opt_out: int = Field(..., description="The number of opt outs")
    help: int = Field(..., description="The number of help messages received")


class WhatsAppMessageStatistics(BrazeBaseModel):
    """
    Model representing WhatsApp message statistics.
    """

    variation_api_id: str = Field(..., description="The variation API identifier")
    sent: int = Field(..., description="The number of sends")
    delivered: int = Field(..., description="The number of delivered messages")
    failed: int = Field(..., description="The number of failed deliveries")
    read: int = Field(..., description="The number of opened messages")


class ContentCardMessageStatistics(BrazeBaseModel):
    """
    Model representing content card message statistics.
    """

    variation_api_id: str = Field(..., description="The variation API identifier")
    sent: int = Field(..., description="The number of sends")
    total_clicks: int = Field(..., description="The number of total clicks")
    total_dismissals: int = Field(..., description="The number of total dismissals")
    total_impressions: int = Field(..., description="The number of total impressions")
    unique_clicks: int = Field(..., description="The number of unique clicks")
    unique_dismissals: int = Field(..., description="The number of unique dismissals")
    unique_impressions: int = Field(..., description="The number of unique impressions")


class InAppMessageStatistics(BrazeBaseModel):
    """
    Model representing in-app message statistics.
    """

    variation_api_id: str = Field(..., description="The variation API identifier")
    impressions: int = Field(..., description="The number of impressions")
    clicks: int = Field(..., description="The number of clicks")
    first_button_clicks: int = Field(..., description="The number of first button clicks")
    second_button_clicks: int = Field(..., description="The number of second button clicks")


class KindlePushMessageStatistics(BrazeBaseModel):
    """
    Model representing Kindle push message statistics.
    """

    variation_api_id: str = Field(..., description="The variation API identifier")
    sent: int = Field(..., description="The number of sends")
    direct_opens: int = Field(..., description="The number of direct opens")
    total_opens: int = Field(..., description="The number of total opens")
    bounces: int = Field(..., description="The number of bounces")
    body_clicks: int = Field(..., description="The number of body clicks")


class WebPushMessageStatistics(BrazeBaseModel):
    """
    Model representing web push message statistics.
    """

    variation_api_id: str = Field(..., description="The variation API identifier")
    sent: int = Field(..., description="The number of sends")
    direct_opens: int = Field(..., description="The number of direct opens")
    total_opens: int = Field(..., description="The number of total opens")
    bounces: int = Field(..., description="The number of bounces")
    body_clicks: int = Field(..., description="The number of body clicks")


class TriggerInAppMessageStatistics(BrazeBaseModel):
    """
    Model representing trigger in-app message statistics for multivariate campaigns.
    """

    variation_name: str | None = Field(None, description="The variation name")
    impressions: int | None = Field(None, description="The number of impressions")
    clicks: int | None = Field(None, description="The number of clicks")
    first_button_clicks: int | None = Field(None, description="The number of first button clicks")
    second_button_clicks: int | None = Field(None, description="The number of second button clicks")
    revenue: float = Field(..., description="The number of dollars of revenue (USD)")
    unique_recipients: int = Field(..., description="The number of unique recipients")
    conversions: int = Field(..., description="The number of conversions")
    conversions_by_send_time: int = Field(
        ..., description="The number of conversions attributed to the date the campaign was sent"
    )
    conversions1: int | None = Field(
        None, description="The number of conversions for the second conversion event"
    )
    conversions1_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the second conversion event attributed to the date the campaign was sent",
    )
    conversions2: int | None = Field(
        None, description="The number of conversions for the third conversion event"
    )
    conversions2_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the third conversion event attributed to the date the campaign was sent",
    )
    conversions3: int | None = Field(
        None, description="The number of conversions for the fourth conversion event"
    )
    conversions3_by_send_time: int | None = Field(
        None,
        description="The number of conversions for the fourth conversion event attributed to the date the campaign was sent",
    )
    enrolled: int | None = Field(None, description="The number of enrolled users")


class MessageStatistics(BrazeBaseModel):
    """
    Model representing message statistics by channel for both campaign and send data series.
    """

    ios_push: list[IOSPushMessageStatistics] | None = Field(None, description="IOS push messages")
    android_push: list[AndroidPushMessageStatistics] | None = Field(
        None, description="Android push messages"
    )
    kindle_push: list[KindlePushMessageStatistics] | None = Field(
        None, description="Kindle push messages"
    )
    web_push: list[WebPushMessageStatistics] | None = Field(None, description="Web push messages")
    webhook: list[WebhookMessageStatistics] | None = Field(None, description="Webhook messages")
    email: list[EmailMessageStatistics] | None = Field(None, description="Email messages")
    sms: list[SMSMessageStatistics] | None = Field(None, description="SMS messages")
    whats_app: list[WhatsAppMessageStatistics] | None = Field(None, description="WhatsApp messages")
    content_cards: list[ContentCardMessageStatistics] | None = Field(
        None, description="Content card messages"
    )
    in_app_message: list[InAppMessageStatistics] | None = Field(None, description="In-app messages")
    trigger_in_app_message: list[TriggerInAppMessageStatistics] | None = Field(
        None, description="Trigger in-app messages for multivariate campaigns"
    )



from unittest.mock import patch

import pytest

from braze_mcp.models import ScheduledBroadcastsResponse
from braze_mcp.tools.messages import get_scheduled_broadcasts


@pytest.fixture
def sample_scheduled_broadcasts_response():
    """Sample scheduled broadcasts API response"""
    return {
        "scheduled_broadcasts": [
            {
                "name": "Test Campaign",
                "id": "campaign_123",
                "type": "Campaign",
                "tags": ["test"],
                "next_send_time": "2024-02-01T10:00:00Z",
                "schedule_type": "local_time_zones",
            }
        ]
    }


class TestGetScheduledBroadcasts:
    """Test get_scheduled_broadcasts function"""

    @pytest.mark.asyncio
    async def test_success_response(
        self, mock_context, mock_braze_context, sample_scheduled_broadcasts_response
    ):
        """Test successful scheduled broadcasts retrieval."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch("braze_mcp.tools.messages.get_braze_context", return_value=mock_braze_context),
            patch(
                "braze_mcp.tools.messages.make_request",
                return_value=SuccessResponse(data=sample_scheduled_broadcasts_response, headers={}),
            ) as mock_request,
        ):
            result = await get_scheduled_broadcasts(mock_context, end_time="2024-02-10T00:00:00Z")

            assert isinstance(result, ScheduledBroadcastsResponse)
            assert len(result.scheduled_broadcasts) == 1

            broadcast = result.scheduled_broadcasts[0]
            assert broadcast.name == "Test Campaign"
            assert broadcast.type == "Campaign"

            # Verify request parameters
            call_args = mock_request.call_args
            assert call_args[0][2] == "messages/scheduled_broadcasts"
            assert call_args[0][3]["end_time"] == "2024-02-10T00:00:00Z"

    @pytest.mark.asyncio
    async def test_error_response(self, mock_context, mock_braze_context):
        """Test error response handling."""
        from braze_mcp.utils.http import FailureResponse

        with (
            patch("braze_mcp.tools.messages.get_braze_context", return_value=mock_braze_context),
            patch(
                "braze_mcp.tools.messages.make_request",
                return_value=FailureResponse(
                    data={"error": "Invalid format"}, error=Exception("Invalid format")
                ),
            ),
        ):
            result = await get_scheduled_broadcasts(mock_context, end_time="invalid")

            assert isinstance(result, dict)
            assert "error" in result



from pydantic import BaseModel, ConfigDict, Field


class BrazeBaseModel(BaseModel):
    """Base model for all Braze MCP models with extra fields allowed.

    This allows models to accept additional fields in case the Braze API
    changes response format and includes new fields not yet defined in our models.
    """

    model_config = ConfigDict(extra="allow")


class PaginationInfo(BrazeBaseModel):
    """Model representing pagination information for paginated responses."""

    current_page_count: int = Field(..., description="Number of items in the current page")
    has_more_pages: bool = Field(..., description="Whether there are more pages available")
    next_cursor: str | None = Field(None, description="Cursor for the next page, if available")
    max_per_page: int = Field(..., description="Maximum number of items per page")
    link_header: str | None = Field(None, description="Raw Link header from the API response")



from typing import Any

from mcp.server.fastmcp import Context

from braze_mcp.models import SessionDataSeriesResponse
from braze_mcp.utils import get_braze_context, get_logger, make_request
from braze_mcp.utils.http import (
    handle_response,
)

__register_mcp_tools__ = True

logger = get_logger(__name__)


async def get_session_data_series(
    ctx: Context,
    length: int,
    unit: str | None = None,
    ending_at: str | None = None,
    app_id: str | None = None,
    segment_id: str | None = None,
) -> SessionDataSeriesResponse | dict[str, Any]:
    """Use this endpoint to retrieve a series of the number of sessions for your app over a designated time period.

    Args:
        ctx: The MCP context
        length: Maximum number of units (days or hours) before ending_at to include in the returned series. Must be between 1 and 100 (inclusive)
        unit: Unit of time between data points. Can be day or hour, defaults to day
        ending_at: Date on which the data series should end (ISO-8601 string). Defaults to time of the request
        app_id: App API identifier retrieved from the API Keys page to limit analytics to a specific app
        segment_id: Segment API identifier. Segment ID indicating the analytics-enabled segment for which sessions should be returned

    Returns:
        SessionDataSeriesResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    url_path = "sessions/data_series"

    params = {
        "length": length,
        "unit": unit,
        "ending_at": ending_at,
        "app_id": app_id,
        "segment_id": segment_id,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, SessionDataSeriesResponse, "fetch session data series", logger)



"""Real API validation tests for canvases tools."""

import pytest

from braze_mcp.models.canvases import CanvasListResponse
from braze_mcp.tools.canvases import get_canvas_list


@pytest.mark.real_api
class TestCanvasesRealAPI:
    """Real API tests for canvases tools."""

    @pytest.mark.asyncio
    async def test_get_canvas_list_basic(self, real_context):
        """Test get_canvas_list against real Braze API."""
        result = await get_canvas_list(real_context)

        # Validate return type is Pydantic model
        assert isinstance(result, CanvasListResponse), (
            f"Expected CanvasListResponse, got {type(result)}"
        )

        # Validate response structure
        assert hasattr(result, "canvases"), "Response should have canvases attribute"
        assert hasattr(result, "message"), "Response should have message attribute"

        # Validate data types
        assert isinstance(result.canvases, list), "Canvases should be a list"
        assert isinstance(result.message, str), "Message should be a string"

        # If canvases exist, validate structure
        if result.canvases:
            canvas = result.canvases[0]
            assert hasattr(canvas, "id"), "Canvas should have id attribute"
            assert hasattr(canvas, "name"), "Canvas should have name attribute"
            assert hasattr(canvas, "last_edited"), "Canvas should have last_edited attribute"

    @pytest.mark.asyncio
    async def test_get_canvas_list_with_parameters(self, real_context):
        """Test get_canvas_list with parameters against real Braze API."""
        result = await get_canvas_list(
            real_context, page=0, include_archived=False, sort_direction="desc"
        )

        # Validate return type is Pydantic model
        assert isinstance(result, CanvasListResponse), (
            f"Expected CanvasListResponse, got {type(result)}"
        )

        # Should still get valid response
        assert hasattr(result, "canvases"), "Response should have canvases attribute"
        assert hasattr(result, "message"), "Response should have message attribute"



from unittest.mock import patch

import pytest

from braze_mcp.models import (
    SegmentDataSeriesResponse,
    SegmentDetails,
    SegmentListResponse,
)
from braze_mcp.tools.segments import (
    get_segment_data_series,
    get_segment_details,
    get_segment_list,
)


@pytest.fixture
def sample_segment_list_data():
    return {
        "message": "success",
        "segments": [
            {
                "id": "segment_id_1",
                "name": "Test Segment 1",
                "analytics_tracking_enabled": True,
                "tags": ["tag1", "tag2"],
            },
            {
                "id": "segment_id_2",
                "name": "Test Segment 2",
                "analytics_tracking_enabled": False,
                "tags": ["tag3"],
            },
        ],
    }


@pytest.fixture
def sample_segment_data_series_data():
    return {
        "message": "success",
        "data": [
            {"time": "2025-01-01", "size": 10000},
            {"time": "2025-01-02", "size": 10500},
        ],
    }


@pytest.fixture
def sample_segment_details_data():
    return {
        "message": "success",
        "created_at": "2025-01-01T10:00:00Z",
        "updated_at": "2025-01-01T12:00:00Z",
        "name": "Test Segment Details",
        "description": "Users who have made a purchase",
        "text_description": "Segment for analyzing purchase behavior",
        "tags": ["analytics", "purchase"],
        "teams": ["Marketing", "Analytics"],
    }


class TestGetSegmentList:
    """Test get_segment_list function"""

    @pytest.mark.asyncio
    async def test_get_segment_list_success_with_default_parameters(
        self, mock_context, mock_braze_context, sample_segment_list_data
    ):
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=SuccessResponse(data=sample_segment_list_data, headers={}),
            ) as mock_request:
                result = await get_segment_list(mock_context)

                assert isinstance(result, SegmentListResponse)
                assert result.message == "success"
                assert len(result.segments) == 2
                assert result.segments[0].id == "segment_id_1"
                assert result.segments[0].name == "Test Segment 1"
                assert result.segments[0].analytics_tracking_enabled
                assert len(result.segments[0].tags) == 2
                assert result.segments[1].id == "segment_id_2"
                assert not result.segments[1].analytics_tracking_enabled

                call_args = mock_request.call_args
                assert call_args[0][0] == mock_braze_context.http_client
                assert call_args[0][1] == mock_braze_context.base_url
                assert call_args[0][2] == "segments/list"
                params = call_args[0][3]
                assert params["page"] == 0
                assert params["sort_direction"] == "desc"

    @pytest.mark.asyncio
    async def test_get_segment_list_success_with_custom_parameters(
        self, mock_context, mock_braze_context, sample_segment_list_data
    ):
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=SuccessResponse(data=sample_segment_list_data, headers={}),
            ) as mock_request:
                await get_segment_list(mock_context, page=2, sort_direction="desc")

                call_args = mock_request.call_args
                params = call_args[0][3]
                assert params["page"] == 2
                assert params["sort_direction"] == "desc"

    @pytest.mark.asyncio
    async def test_get_segment_list_request_failure(self, mock_context, mock_braze_context):
        from braze_mcp.utils.http import FailureResponse

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=FailureResponse(
                    data={"error": "Request failed"},
                    error=Exception("Request failed"),
                ),
            ):
                result = await get_segment_list(mock_context)

                assert isinstance(result, dict)
                assert "error" in result
                assert result["error"] == "Request failed"

    @pytest.mark.asyncio
    async def test_get_segment_list_response_fails_parsing(self, mock_context, mock_braze_context):
        """Test get segment list returns the raw response when the response fails schema validation"""
        invalid_data = {"message": "success", "segments": [{"invalid": "data"}]}
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=SuccessResponse(data=invalid_data, headers={}),
            ):
                with patch("braze_mcp.tools.segments.logger.exception") as mock_logger:
                    result = await get_segment_list(mock_context)

                    assert isinstance(result, dict)
                    assert result == invalid_data
                    mock_logger.assert_called_once()


class TestGetSegmentDataSeries:
    """Test get_segment_data_series function"""

    @pytest.mark.asyncio
    async def test_get_segment_data_series_success_with_required_parameters(
        self, mock_context, mock_braze_context, sample_segment_data_series_data
    ):
        from braze_mcp.utils.http import SuccessResponse

        segment_id = "test_segment_id"

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=SuccessResponse(data=sample_segment_data_series_data, headers={}),
            ) as mock_request:
                result = await get_segment_data_series(
                    mock_context, segment_id=segment_id, length=14
                )

                assert isinstance(result, SegmentDataSeriesResponse)
                assert result.message == "success"
                assert len(result.data) == 2
                assert result.data[0].time == "2025-01-01"
                assert result.data[0].size == 10000
                assert result.data[1].time == "2025-01-02"
                assert result.data[1].size == 10500

                call_args = mock_request.call_args
                assert call_args[0][0] == mock_braze_context.http_client
                assert call_args[0][1] == mock_braze_context.base_url
                assert call_args[0][2] == "segments/data_series"
                params = call_args[0][3]
                assert params["segment_id"] == segment_id
                assert params["length"] == 14
                assert params["ending_at"] is None

    @pytest.mark.asyncio
    async def test_get_segment_data_series_success_with_all_parameters(
        self, mock_context, mock_braze_context, sample_segment_data_series_data
    ):
        from braze_mcp.utils.http import SuccessResponse

        segment_id = "test_segment_id"

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=SuccessResponse(data=sample_segment_data_series_data, headers={}),
            ) as mock_request:
                await get_segment_data_series(
                    mock_context,
                    segment_id=segment_id,
                    length=30,
                    ending_at="2025-01-02T23:59:59-05:00",
                )

                call_args = mock_request.call_args
                params = call_args[0][3]
                assert params["segment_id"] == segment_id
                assert params["length"] == 30
                assert params["ending_at"] == "2025-01-02T23:59:59-05:00"

    @pytest.mark.asyncio
    async def test_get_segment_data_series_request_failure(self, mock_context, mock_braze_context):
        from braze_mcp.utils.http import FailureResponse

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=FailureResponse(
                    data={"error": "Request failed"},
                    error=Exception("Request failed"),
                ),
            ):
                result = await get_segment_data_series(
                    mock_context, segment_id="test_id", length=14
                )

                assert isinstance(result, dict)
                assert "error" in result


class TestGetSegmentDetails:
    """Test get_segment_details function"""

    @pytest.mark.asyncio
    async def test_get_segment_details_success(
        self, mock_context, mock_braze_context, sample_segment_details_data
    ):
        from braze_mcp.utils.http import SuccessResponse

        segment_id = "test_segment_id"

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=SuccessResponse(data=sample_segment_details_data, headers={}),
            ) as mock_request:
                result = await get_segment_details(mock_context, segment_id=segment_id)

                assert isinstance(result, SegmentDetails)
                assert result.message == "success"
                assert result.name == "Test Segment Details"
                assert result.description == "Users who have made a purchase"
                assert result.text_description == "Segment for analyzing purchase behavior"
                assert len(result.tags) == 2
                assert "analytics" in result.tags
                assert "purchase" in result.tags
                assert len(result.teams) == 2
                assert "Marketing" in result.teams
                assert "Analytics" in result.teams

                call_args = mock_request.call_args
                assert call_args[0][0] == mock_braze_context.http_client
                assert call_args[0][1] == mock_braze_context.base_url
                assert call_args[0][2] == "segments/details"
                params = call_args[0][3]
                assert params["segment_id"] == segment_id

    @pytest.mark.asyncio
    async def test_get_segment_details_request_failure(self, mock_context, mock_braze_context):
        from braze_mcp.utils.http import FailureResponse

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=FailureResponse(
                    data={"error": "Request failed"},
                    error=Exception("Request failed"),
                ),
            ):
                result = await get_segment_details(mock_context, segment_id="test_segment_id")

                assert isinstance(result, dict)
                assert "error" in result

    @pytest.mark.asyncio
    async def test_get_segment_details_response_fails_parsing(
        self, mock_context, mock_braze_context
    ):
        invalid_data = {"message": "success", "invalid": "data"}
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.segments.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.segments.make_request",
                return_value=SuccessResponse(data=invalid_data, headers={}),
            ):
                with patch("braze_mcp.tools.segments.logger.exception") as mock_logger:
                    result = await get_segment_details(mock_context, segment_id="test_segment_id")

                    assert isinstance(result, dict)
                    assert result == invalid_data
                    mock_logger.assert_called_once()



from typing import Any

from mcp.server.fastmcp import Context

from braze_mcp.models.canvas_dataseries import CanvasDataSeriesResponse
from braze_mcp.models.canvases import (
    CanvasDataSummaryResponse,
    CanvasDetails,
    CanvasListResponse,
)
from braze_mcp.models.errors import validation_error
from braze_mcp.utils import get_braze_context, get_logger, make_request
from braze_mcp.utils.http import (
    handle_response,
)

__register_mcp_tools__ = True

logger = get_logger(__name__)


async def get_canvas_list(
    ctx: Context,
    page: int = 0,
    include_archived: bool = False,
    sort_direction: str = "desc",
    last_edit_time_gt: str | None = None,
) -> CanvasListResponse | dict[str, Any]:
    """Use this endpoint to export a list of Canvases, each of which will include its name,
    Canvas API identifier, and tags associated with the Canvas.

    Args:
        ctx: The MCP context
        page: Represents the page of Canvases to return, defaults to 0 (returns the first set of up to 100)
        include_archived: Whether to include archived Canvases, defaults to false
        sort_direction: Sort creation time - 'desc' for newest to oldest, 'asc' for oldest to newest
        last_edit_time_gt: Filters results to Canvases edited after this time (format: yyyy-MM-DDTHH:mm:ss)

    Returns:
        CanvasListResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    url_path = "canvas/list"

    params = {
        "page": page,
        "include_archived": include_archived,
        "sort_direction": sort_direction,
        "last_edit.time[gt]": last_edit_time_gt,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, CanvasListResponse, "fetch canvases", logger)


async def get_canvas_details(
    ctx: Context, canvas_id: str, post_launch_draft_version: bool = False
) -> CanvasDetails | dict[str, Any]:
    """Use this endpoint to export metadata about a Canvas, such as the name, time created, current status, and more.

    Args:
        ctx: The MCP context
        canvas_id: The Canvas API identifier - can be found on the API Keys page and the Canvas Details page within your dashboard; or you can use the Export canvases list endpoint.
        post_launch_draft_version: For Canvases that have a post-launch draft, setting this to true will show any draft changes available. Defaults to false

    Returns:
        CanvasDetails when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    url_path = "canvas/details"

    params = {
        "canvas_id": canvas_id,
        "post_launch_draft_version": post_launch_draft_version,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, CanvasDetails, "fetch canvas details", logger)


async def get_canvas_data_summary(
    ctx: Context,
    canvas_id: str,
    ending_at: str,
    starting_at: str | None = None,
    length: int | None = None,
    include_variant_breakdown: bool = False,
    include_step_breakdown: bool = False,
    include_deleted_step_data: bool = False,
) -> CanvasDataSummaryResponse | dict[str, Any]:
    """Use this endpoint to export rollups of time series data for a Canvas, providing a concise summary of Canvas results.

    Args:
        ctx: The MCP context
        canvas_id: The Canvas API identifier - can be found on the API Keys page and the Canvas Details page within your dashboard; or you can use the Export canvases list endpoint.
        ending_at: Date on which the data export should end (ISO-8601 datetime string). Defaults to time of the request.
        starting_at: Date on which the data export should begin (ISO-8601 datetime string). Either length or starting_at is required.
        length: Maximum number of days before ending_at to include in the returned series. Must be between 1 and 14 (inclusive). Either length or starting_at is required.
        include_variant_breakdown: Whether to include variant statistics (defaults to false).
        include_step_breakdown: Whether to include step statistics (defaults to false).
        include_deleted_step_data: Whether to include step statistics for deleted steps (defaults to false).

    Returns:
        CanvasDataSummaryResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    # Validate that either starting_at or length is provided
    if starting_at is None and length is None:
        return validation_error(
            "Either starting_at or length parameter is required", "get_canvas_data_summary"
        )

    url_path = "canvas/data_summary"

    params = {
        "canvas_id": canvas_id,
        "ending_at": ending_at,
        "starting_at": starting_at,
        "length": length,
        "include_variant_breakdown": include_variant_breakdown,
        "include_step_breakdown": include_step_breakdown,
        "include_deleted_step_data": include_deleted_step_data,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, CanvasDataSummaryResponse, "fetch canvas data summary", logger)


async def get_canvas_data_series(
    ctx: Context,
    canvas_id: str,
    ending_at: str,
    starting_at: str | None = None,
    length: int | None = None,
    include_variant_breakdown: bool = False,
    include_step_breakdown: bool = False,
    include_deleted_step_data: bool = False,
) -> CanvasDataSeriesResponse | dict[str, Any]:
    """Use this endpoint to export time series data for a Canvas.

    Args:
        ctx: The MCP context
        canvas_id: The Canvas API identifier - can be found on the API Keys page and the Canvas Details page within your dashboard; or you can use the Export canvases list endpoint.
        ending_at: Date on which the data export should end (ISO-8601 datetime string). Defaults to time of the request.
        starting_at: Date on which the data export should begin (ISO-8601 datetime string). Either length or starting_at is required.
        length: Maximum number of days before ending_at to include in the returned series. Must be between 1 and 14 (inclusive). Either length or starting_at is required.
        include_variant_breakdown: Whether to include variant statistics (defaults to false).
        include_step_breakdown: Whether to include step statistics (defaults to false).
        include_deleted_step_data: Whether to include step statistics for deleted steps (defaults to false).

    Returns:
        CanvasDataSeriesResponse when parsing succeeds, otherwise a dictionary containing the raw response data.
    """
    # Validate that either starting_at or length is provided
    if starting_at is None and length is None:
        return validation_error(
            "Either starting_at or length parameter is required", operation="get_canvas_data_series"
        )

    url_path = "canvas/data_series"

    params = {
        "canvas_id": canvas_id,
        "ending_at": ending_at,
        "starting_at": starting_at,
        "length": length,
        "include_variant_breakdown": include_variant_breakdown,
        "include_step_breakdown": include_step_breakdown,
        "include_deleted_step_data": include_deleted_step_data,
    }

    bctx = get_braze_context(ctx)

    response = await make_request(bctx.http_client, bctx.base_url, url_path, params)

    return handle_response(response, CanvasDataSeriesResponse, "fetch canvas data series", logger)



from unittest.mock import patch

import pytest

from braze_mcp.models import PreferenceCenterDetails, PreferenceCentersResponse
from braze_mcp.tools.preference_centers import get_preference_center_details, get_preference_centers


@pytest.fixture
def sample_preference_centers_data():
    """Sample data for preference centers endpoint."""
    return {
        "preference_centers": [
            {
                "name": "My Preference Center 1",
                "preference_center_api_id": "preference_center_api_id_1",
                "created_at": "2022-08-17T15:46:10Z",
                "updated_at": "2022-08-17T15:46:10Z",
            },
            {
                "name": "My Preference Center 2",
                "preference_center_api_id": "preference_center_api_id_2",
                "created_at": "2022-08-19T11:13:06Z",
                "updated_at": "2022-08-19T11:13:06Z",
            },
            {
                "name": "My Preference Center 3",
                "preference_center_api_id": "preference_center_api_id_3",
                "created_at": "2022-08-19T11:30:50Z",
                "updated_at": "2022-08-19T11:30:50Z",
            },
            {
                "name": "My Preference Center 4",
                "preference_center_api_id": "preference_center_api_id_4",
                "created_at": "2022-09-13T20:41:34Z",
                "updated_at": "2022-09-13T20:41:34Z",
            },
        ]
    }


@pytest.fixture
def empty_preference_centers_data():
    """Sample data for empty preference centers response."""
    return {"preference_centers": []}


@pytest.fixture
def sample_preference_center_details_data():
    """Sample data for preference center details endpoint."""
    return {
        "name": "My Preference Center",
        "preference_center_api_id": "preference_center_api_id_123",
        "created_at": "2022-08-17T15:46:10Z",
        "updated_at": "2022-08-17T15:46:10Z",
        "preference_center_title": "Example preference center title",
        "preference_center_page_html": "<html><body>HTML for preference center here</body></html>",
        "confirmation_page_html": "<html><body>HTML for confirmation page here</body></html>",
        "redirect_page_html": None,
        "preference_center_options": {
            "meta-viewport-content": "width=device-width, initial-scale=2"
        },
        "state": "active",
    }


class TestGetPreferenceCenters:
    """Test get_preference_centers function"""

    @pytest.mark.asyncio
    async def test_success_response_with_data(
        self, mock_context, mock_braze_context, sample_preference_centers_data
    ):
        """Test successful preference centers retrieval with comprehensive pydantic model validation."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.preference_centers.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.preference_centers.make_request",
                return_value=SuccessResponse(data=sample_preference_centers_data, headers={}),
            ) as mock_request,
        ):
            result = await get_preference_centers(mock_context)

            # Validate pydantic model structure
            assert isinstance(result, PreferenceCentersResponse)
            assert hasattr(result, "preference_centers")

            # Validate preference centers structure
            assert len(result.preference_centers) == 4
            assert isinstance(result.preference_centers, list)

            # Verify request parameters
            call_args = mock_request.call_args
            assert call_args[0][2] == "preference_center/v1/list"
            assert call_args[0][0] == mock_braze_context.http_client
            assert call_args[0][1] == mock_braze_context.base_url
            # Verify no parameters are passed (empty dict)
            assert call_args[0][3] == {}

    @pytest.mark.asyncio
    async def test_success_response_empty_list(
        self, mock_context, mock_braze_context, empty_preference_centers_data
    ):
        """Test successful preference centers retrieval with empty list."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.preference_centers.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.preference_centers.make_request",
                return_value=SuccessResponse(data=empty_preference_centers_data, headers={}),
            ) as mock_request,
        ):
            result = await get_preference_centers(mock_context)

            # Validate pydantic model structure
            assert isinstance(result, PreferenceCentersResponse)
            assert hasattr(result, "preference_centers")
            assert isinstance(result.preference_centers, list)
            assert len(result.preference_centers) == 0

            # Verify request was made correctly
            call_args = mock_request.call_args
            assert call_args[0][2] == "preference_center/v1/list"
            assert call_args[0][3] == {}

    @pytest.mark.asyncio
    async def test_error_response(self, mock_context, mock_braze_context):
        """Test preference centers error handling."""
        from braze_mcp.utils.http import FailureResponse

        with (
            patch(
                "braze_mcp.tools.preference_centers.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.preference_centers.make_request",
                return_value=FailureResponse(
                    data={"error": "Unauthorized"}, error=Exception("API Error")
                ),
            ),
        ):
            result = await get_preference_centers(mock_context)

            # Should return error dict, not pydantic model
            assert isinstance(result, dict)
            assert "error" in result
            assert result["error"] == "Unauthorized"

    @pytest.mark.asyncio
    async def test_parsing_failure_fallback(self, mock_context, mock_braze_context):
        """Test that malformed response falls back to raw dict."""
        from braze_mcp.utils.http import SuccessResponse

        # Invalid response structure that will fail pydantic parsing
        invalid_data = {"invalid_field": "should cause parsing failure"}

        with (
            patch(
                "braze_mcp.tools.preference_centers.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.preference_centers.make_request",
                return_value=SuccessResponse(data=invalid_data, headers={}),
            ),
        ):
            result = await get_preference_centers(mock_context)

            # Should return raw dict when pydantic parsing fails
            assert isinstance(result, dict)
            assert result == invalid_data


class TestGetPreferenceCenterDetails:
    """Test get_preference_center_details function"""

    @pytest.mark.asyncio
    async def test_success_response_with_data(
        self, mock_context, mock_braze_context, sample_preference_center_details_data
    ):
        """Test successful preference center details retrieval with comprehensive pydantic model validation."""
        from braze_mcp.utils.http import SuccessResponse

        preference_center_id = "preference_center_api_id_123"

        with (
            patch(
                "braze_mcp.tools.preference_centers.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.preference_centers.make_request",
                return_value=SuccessResponse(
                    data=sample_preference_center_details_data, headers={}
                ),
            ) as mock_request,
        ):
            result = await get_preference_center_details(mock_context, preference_center_id)

            # Validate pydantic model structure
            assert isinstance(result, PreferenceCenterDetails)

            # Validate specific values
            assert result.name == "My Preference Center"
            assert result.preference_center_api_id == "preference_center_api_id_123"
            assert result.created_at == "2022-08-17T15:46:10Z"
            assert result.updated_at == "2022-08-17T15:46:10Z"
            assert result.preference_center_title == "Example preference center title"
            assert result.state == "active"
            assert result.redirect_page_html is None

            # Validate HTML content exists
            assert "<html>" in result.preference_center_page_html
            assert "<html>" in result.confirmation_page_html

            # Validate options structure
            assert isinstance(result.preference_center_options, dict)
            assert "meta-viewport-content" in result.preference_center_options
            assert (
                result.preference_center_options["meta-viewport-content"]
                == "width=device-width, initial-scale=2"
            )

            # Verify request parameters
            call_args = mock_request.call_args
            assert call_args[0][2] == f"preference_center/v1/{preference_center_id}"
            assert call_args[0][0] == mock_braze_context.http_client
            assert call_args[0][1] == mock_braze_context.base_url
            # Verify no additional parameters are passed
            assert call_args[0][3] == {}

    @pytest.mark.asyncio
    async def test_error_response(self, mock_context, mock_braze_context):
        """Test preference center details error handling."""
        from braze_mcp.utils.http import FailureResponse

        preference_center_id = "invalid_id"

        with (
            patch(
                "braze_mcp.tools.preference_centers.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.preference_centers.make_request",
                return_value=FailureResponse(
                    data={"error": "Preference center not found"}, error=Exception("Not Found")
                ),
            ),
        ):
            result = await get_preference_center_details(mock_context, preference_center_id)

            # Should return error dict, not pydantic model
            assert isinstance(result, dict)
            assert "error" in result
            assert result["error"] == "Preference center not found"

    @pytest.mark.asyncio
    async def test_parsing_failure_fallback(self, mock_context, mock_braze_context):
        """Test that malformed response falls back to raw dict."""
        from braze_mcp.utils.http import SuccessResponse

        # Invalid response structure that will fail pydantic parsing
        invalid_data = {"invalid_field": "missing required fields"}

        with (
            patch(
                "braze_mcp.tools.preference_centers.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.preference_centers.make_request",
                return_value=SuccessResponse(data=invalid_data, headers={}),
            ),
        ):
            result = await get_preference_center_details(mock_context, "test_id")

            # Should return raw dict when pydantic parsing fails
            assert isinstance(result, dict)
            assert result == invalid_data



import pytest


@pytest.fixture
def sample_events_list_data():
    return {
        "message": "success",
        "events": ["Event A", "Event B", "Event C"],
    }


@pytest.fixture
def sample_events_data_series_data():
    return {
        "message": "success",
        "data": [
            {"time": "2024-01-01", "count": 100},
            {"time": "2024-01-02", "count": 150},
            {"time": "2024-01-03", "count": 120},
        ],
    }


@pytest.fixture
def sample_events_data():
    return {
        "message": "success",
        "events": [
            {
                "name": "Test Event 1",
                "description": "Description for test event 1",
                "included_in_analytics_report": True,
                "status": "Active",
                "tag_names": ["Tag One", "Tag Two"],
            },
            {
                "name": "Test Event 2",
                "description": "Description for test event 2",
                "included_in_analytics_report": False,
                "status": "Inactive",
                "tag_names": ["Tag Three"],
            },
        ],
    }



from unittest.mock import MagicMock, patch

import pytest

from braze_mcp.tools.custom_attributes import get_custom_attributes


@pytest.fixture
def sample_custom_attributes_data():
    return {
        "message": "success",
        "attributes": [
            {
                "array_length": None,
                "data_type": "String",
                "description": "User's first name",
                "name": "first_name",
                "status": "Active",
                "tag_names": ["user_data", "profile"],
            },
            {
                "array_length": 10,
                "data_type": "Array",
                "description": "User's preferences",
                "name": "preferences",
                "status": "Active",
                "tag_names": ["user_data"],
            },
            {
                "array_length": None,
                "data_type": "Boolean",
                "description": None,
                "name": "is_premium",
                "status": "Blocklisted",
                "tag_names": [],
            },
        ],
    }


@pytest.fixture
def sample_response_headers():
    return {
        "content-type": "application/json",
        "link": '<https://test.braze.com/custom_attributes/?cursor=abc123>; rel="next"',
    }


class TestGetCustomAttributes:
    """Test get_custom_attributes function"""

    @pytest.mark.asyncio
    async def test_get_custom_attributes_success(
        self,
        mock_context,
        mock_braze_context,
        sample_custom_attributes_data,
        sample_response_headers,
    ):
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.custom_attributes.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.custom_attributes.make_request",
                return_value=SuccessResponse(
                    data=sample_custom_attributes_data,
                    headers=sample_response_headers,
                ),
            ) as mock_request:
                with patch(
                    "braze_mcp.tools.custom_attributes.extract_cursor_from_link_header",
                    return_value="abc123",
                ):
                    result = await get_custom_attributes(mock_context)

                    from braze_mcp.models import CustomAttributesWithPagination

                    assert isinstance(result, CustomAttributesWithPagination)
                    assert result.message == "success"
                    assert len(result.attributes) == 3
                    assert result.attributes[0].name == "first_name"
                    assert result.attributes[1].name == "preferences"
                    assert result.attributes[2].name == "is_premium"

                    # Check pagination info
                    pagination = result.pagination_info
                    assert pagination.current_page_count == 3
                    assert pagination.has_more_pages
                    assert pagination.next_cursor == "abc123"
                    assert pagination.max_per_page == 50
                    assert pagination.link_header == sample_response_headers["link"]

                    # Verify call was made correctly - new signature: client, base_url, url_path, params
                    call_args = mock_request.call_args
                    assert call_args[0][0] == mock_braze_context.http_client  # client
                    assert call_args[0][1] == mock_braze_context.base_url  # base_url
                    assert call_args[0][2] == "custom_attributes"  # url_path
                    params = call_args[0][3]  # params
                    assert params == {}

    @pytest.mark.asyncio
    async def test_get_custom_attributes_with_cursor(
        self,
        mock_context,
        mock_braze_context,
        sample_custom_attributes_data,
        sample_response_headers,
    ):
        cursor = "test_cursor_123"
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.custom_attributes.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.custom_attributes.make_request",
                return_value=SuccessResponse(
                    data=sample_custom_attributes_data,
                    headers=sample_response_headers,
                ),
            ) as mock_request:
                with patch(
                    "braze_mcp.tools.custom_attributes.extract_cursor_from_link_header",
                    return_value="abc123",
                ):
                    await get_custom_attributes(mock_context, cursor=cursor)

                    # Verify cursor parameter was passed correctly
                    call_args = mock_request.call_args
                    assert call_args[0][0] == mock_braze_context.http_client  # client
                    assert call_args[0][1] == mock_braze_context.base_url  # base_url
                    assert call_args[0][2] == "custom_attributes"  # url_path
                    params = call_args[0][3]  # params
                    assert params["cursor"] == cursor

    @pytest.mark.asyncio
    async def test_get_custom_attributes_no_link_header(
        self, mock_context, mock_braze_context, sample_custom_attributes_data
    ):
        response_headers = {"content-type": "application/json"}
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.custom_attributes.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.custom_attributes.make_request",
                return_value=SuccessResponse(
                    data=sample_custom_attributes_data, headers=response_headers
                ),
            ):
                with patch(
                    "braze_mcp.tools.custom_attributes.extract_cursor_from_link_header",
                    return_value=None,
                ):
                    result = await get_custom_attributes(mock_context)

                    # Check pagination info when no next page
                    pagination = result.pagination_info
                    assert not pagination.has_more_pages
                    assert pagination.next_cursor is None
                    assert pagination.link_header is None

    @pytest.mark.asyncio
    async def test_get_custom_attributes_request_failure(self, mock_context, mock_braze_context):
        from braze_mcp.utils.http import FailureResponse

        with patch(
            "braze_mcp.tools.custom_attributes.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.custom_attributes.make_request",
                return_value=FailureResponse(
                    data={"error": "Request failed"},
                    error=Exception("Request failed"),
                ),
            ):
                result = await get_custom_attributes(mock_context)

                assert isinstance(result, dict)
                assert "error" in result
                assert result["error"] == "Request failed"

    @pytest.mark.asyncio
    async def test_get_custom_attributes_parsing_error(self, mock_context, mock_braze_context):
        """Test get custom attributes returns the raw response when schema validation fails"""
        invalid_data = {"message": "success", "attributes": [{"invalid": "data"}]}
        response_headers = {"content-type": "application/json"}
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.custom_attributes.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.custom_attributes.make_request",
                return_value=SuccessResponse(data=invalid_data, headers=response_headers),
            ):
                with patch("braze_mcp.tools.custom_attributes.logger.exception") as mock_logger:
                    result = await get_custom_attributes(mock_context)

                    assert isinstance(result, dict)
                    assert result == invalid_data
                    mock_logger.assert_called_once()

    @pytest.mark.asyncio
    async def test_get_custom_attributes_empty_attributes(self, mock_context, mock_braze_context):
        empty_data = {"message": "success", "attributes": []}
        response_headers = {"content-type": "application/json"}
        from braze_mcp.utils.http import SuccessResponse

        with patch(
            "braze_mcp.tools.custom_attributes.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.custom_attributes.make_request",
                return_value=SuccessResponse(data=empty_data, headers=response_headers),
            ):
                with patch(
                    "braze_mcp.tools.custom_attributes.extract_cursor_from_link_header",
                    return_value=None,
                ):
                    result = await get_custom_attributes(mock_context)

                    from braze_mcp.models import CustomAttributesWithPagination

                    assert isinstance(result, CustomAttributesWithPagination)
                    assert result.message == "success"
                    assert len(result.attributes) == 0

                    # Check pagination info
                    pagination = result.pagination_info
                    assert pagination.current_page_count == 0
                    assert not pagination.has_more_pages
                    assert pagination.next_cursor is None

    @pytest.mark.asyncio
    async def test_get_custom_attributes_unexpected_response_type(
        self, mock_context, mock_braze_context
    ):
        """Test custom attributes handles unexpected response type"""

        # Create a mock object that isn't SuccessResponse or FailureResponse
        unexpected_response = MagicMock()
        unexpected_response.__class__.__name__ = "UnexpectedResponse"

        with patch(
            "braze_mcp.tools.custom_attributes.get_braze_context",
            return_value=mock_braze_context,
        ):
            with patch(
                "braze_mcp.tools.custom_attributes.make_request",
                return_value=unexpected_response,
            ):
                with patch("braze_mcp.tools.custom_attributes.logger.error") as mock_logger:
                    result = await get_custom_attributes(mock_context)

                    assert isinstance(result, dict)
                    assert "error" in result
                    # Check that we get a standardized error response
                    assert result["success"] is False
                    assert result["error"]["error_type"] == "unexpected_error"
                    mock_logger.assert_called_once_with(
                        f"Unexpected response type: {type(unexpected_response)}"
                    )



from unittest.mock import patch

import pytest

from braze_mcp.tools.integrations import get_integration_job_sync_status, list_integrations


@pytest.fixture
def sample_integrations_data():
    """Sample data for list integrations endpoint."""
    return {
        "results": [
            {
                "integration_id": "12345678-1234-1234-1234-123456789abc",
                "app_group_id": "87654321-4321-4321-4321-cba987654321",
                "integration_name": "Test Integration",
                "integration_type": "snowflake",
                "integration_status": "active",
                "contact_emails": "test@example.com",
                "last_updated_at": "2024-01-15T10:30:00Z",
                "warehouse_type": "snowflake",
                "last_job_start_time": "2024-01-15T09:45:00Z",
                "last_job_status": "success",
                "next_scheduled_run": "2024-01-16T09:45:00Z",
            }
        ],
        "message": "success",
    }


@pytest.fixture
def sample_job_sync_status_data():
    """Sample data for job sync status endpoint."""
    return {
        "results": [
            {
                "job_status": "success",
                "sync_start_time": "2024-01-15T09:45:00Z",
                "sync_finish_time": "2024-01-15T10:30:00Z",
                "last_timestamp_synced": "2024-01-15T09:44:59Z",
                "rows_synced": 1000,
                "rows_failed_with_errors": 0,
            }
        ],
        "message": "success",
    }


class TestListIntegrations:
    """Test list_integrations functionality."""

    @pytest.mark.asyncio
    async def test_list_integrations_success_with_data(
        self, mock_context, mock_braze_context, sample_integrations_data
    ):
        """Test successful list integrations with comprehensive pydantic model validation."""
        from braze_mcp.models import IntegrationsListResponse
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.integrations.get_braze_context", return_value=mock_braze_context
            ),
            patch(
                "braze_mcp.tools.integrations.make_request",
                return_value=SuccessResponse(data=sample_integrations_data, headers={}),
            ) as mock_request,
        ):
            result = await list_integrations(mock_context)

            # Validate pydantic model structure
            assert isinstance(result, IntegrationsListResponse)
            assert hasattr(result, "results")
            assert hasattr(result, "message")

            # Validate response message
            assert result.message == "success"
            assert isinstance(result.message, str)

            # Validate results structure
            assert len(result.results) == 1
            assert isinstance(result.results, list)

            # Validate integration model attributes
            integration = result.results[0]
            required_attrs = [
                "integration_id",
                "app_group_id",
                "integration_name",
                "integration_type",
                "integration_status",
                "contact_emails",
                "last_updated_at",
                "warehouse_type",
                "last_job_start_time",
                "last_job_status",
                "next_scheduled_run",
            ]

            for attr in required_attrs:
                assert hasattr(integration, attr), f"Integration missing required attribute: {attr}"
                assert getattr(integration, attr) is not None, (
                    f"Integration attribute {attr} is None"
                )

            # Validate specific values
            assert integration.integration_id == "12345678-1234-1234-1234-123456789abc"
            assert integration.integration_name == "Test Integration"
            assert integration.integration_type == "snowflake"
            assert integration.integration_status == "active"
            assert integration.warehouse_type == "snowflake"
            assert integration.last_job_status == "success"

            # Verify request parameters
            call_args = mock_request.call_args
            assert call_args[0][2] == "cdi/integrations"
            assert call_args[0][0] == mock_braze_context.http_client
            assert call_args[0][1] == mock_braze_context.base_url

    @pytest.mark.asyncio
    async def test_list_integrations_success_with_cursor(self, mock_context, mock_braze_context):
        """Test list integrations with cursor parameter."""
        from braze_mcp.models import IntegrationsListResponse
        from braze_mcp.utils.http import SuccessResponse

        cursor = "test_cursor"

        with (
            patch(
                "braze_mcp.tools.integrations.get_braze_context", return_value=mock_braze_context
            ),
            patch(
                "braze_mcp.tools.integrations.make_request",
                return_value=SuccessResponse(
                    data={"results": [], "message": "success"}, headers={}
                ),
            ) as mock_request,
        ):
            result = await list_integrations(mock_context, cursor=cursor)

            # Validate pydantic model
            assert isinstance(result, IntegrationsListResponse)
            assert hasattr(result, "results")
            assert hasattr(result, "message")
            assert result.message == "success"
            assert isinstance(result.results, list)
            assert len(result.results) == 0

            # Verify cursor was passed in params
            call_args = mock_request.call_args
            params = call_args[0][3]
            assert params["cursor"] == cursor

    @pytest.mark.asyncio
    async def test_list_integrations_error_response(self, mock_context, mock_braze_context):
        """Test list integrations error handling."""
        from braze_mcp.utils.http import FailureResponse

        with (
            patch(
                "braze_mcp.tools.integrations.get_braze_context", return_value=mock_braze_context
            ),
            patch(
                "braze_mcp.tools.integrations.make_request",
                return_value=FailureResponse(
                    data={"error": "Authentication failed"}, error=Exception("API Error")
                ),
            ),
        ):
            result = await list_integrations(mock_context)

            # Should return error dict, not pydantic model
            assert isinstance(result, dict)
            assert "error" in result
            assert result["error"] == "Authentication failed"


class TestJobSyncStatus:
    """Test integration job sync status functionality."""

    @pytest.mark.asyncio
    async def test_job_sync_status_success_with_data(
        self, mock_context, mock_braze_context, sample_job_sync_status_data
    ):
        """Test successful job sync status with comprehensive pydantic model validation."""
        from braze_mcp.models import JobSyncStatusResponse
        from braze_mcp.utils.http import SuccessResponse

        integration_id = "test-id"

        with (
            patch(
                "braze_mcp.tools.integrations.get_braze_context", return_value=mock_braze_context
            ),
            patch(
                "braze_mcp.tools.integrations.make_request",
                return_value=SuccessResponse(data=sample_job_sync_status_data, headers={}),
            ) as mock_request,
        ):
            result = await get_integration_job_sync_status(mock_context, integration_id)

            # Validate pydantic model structure
            assert isinstance(result, JobSyncStatusResponse)
            assert hasattr(result, "results")
            assert hasattr(result, "message")

            # Validate response message
            assert result.message == "success"
            assert isinstance(result.message, str)

            # Validate results structure
            assert len(result.results) == 1
            assert isinstance(result.results, list)

            # Validate job sync status model attributes
            job_status = result.results[0]
            required_attrs = [
                "job_status",
                "sync_start_time",
                "sync_finish_time",
                "last_timestamp_synced",
                "rows_synced",
                "rows_failed_with_errors",
            ]

            for attr in required_attrs:
                assert hasattr(job_status, attr), (
                    f"JobSyncStatus missing required attribute: {attr}"
                )
                assert getattr(job_status, attr) is not None, (
                    f"JobSyncStatus attribute {attr} is None"
                )

            # Validate specific values and types
            assert job_status.job_status == "success"
            assert isinstance(job_status.job_status, str)
            assert job_status.sync_start_time == "2024-01-15T09:45:00Z"
            assert job_status.sync_finish_time == "2024-01-15T10:30:00Z"
            assert job_status.last_timestamp_synced == "2024-01-15T09:44:59Z"
            assert job_status.rows_synced == 1000
            assert isinstance(job_status.rows_synced, int)
            assert job_status.rows_failed_with_errors == 0
            assert isinstance(job_status.rows_failed_with_errors, int)

            # Verify request parameters
            call_args = mock_request.call_args
            assert call_args[0][2] == "cdi/integrations/test-id/job_sync_status"
            # integration_id is part of the URL path, not the params
            assert "test-id" in call_args[0][2]

    @pytest.mark.asyncio
    async def test_job_sync_status_error_response(self, mock_context, mock_braze_context):
        """Test job sync status error handling."""
        from braze_mcp.utils.http import FailureResponse

        integration_id = "invalid-id"

        with (
            patch(
                "braze_mcp.tools.integrations.get_braze_context", return_value=mock_braze_context
            ),
            patch(
                "braze_mcp.tools.integrations.make_request",
                return_value=FailureResponse(
                    data={"error": "Integration not found"}, error=Exception("API Error")
                ),
            ),
        ):
            result = await get_integration_job_sync_status(mock_context, integration_id)

            # Should return error dict, not pydantic model
            assert isinstance(result, dict)
            assert "error" in result
            assert result["error"] == "Integration not found"



import pytest


@pytest.fixture
def sample_canvas_data():
    return {
        "canvases": [
            {
                "id": "canvas_id_1",
                "name": "Test Canvas 1",
                "last_edited": "2025-01-01T12:00:00Z",
                "tags": [],
            },
            {
                "id": "canvas_id_2",
                "name": "Test Canvas 2",
                "last_edited": "2025-01-02T12:00:00Z",
                "tags": ["tag1", "tag2"],
            },
        ],
        "message": "success",
    }


@pytest.fixture
def sample_canvas_details_data():
    return {
        "message": "success",
        "created_at": "2023-01-01T12:00:00Z",
        "updated_at": "2023-01-10T12:00:00Z",
        "name": "Test Canvas Details",
        "description": "A test canvas for unit testing",
        "archived": False,
        "draft": False,
        "enabled": True,
        "has_post_launch_draft": True,
        "schedule_type": "date",
        "first_entry": "2023-01-01T12:00:00Z",
        "last_entry": "2023-01-10T12:00:00Z",
        "channels": ["email", "push"],
        "variants": [
            {
                "name": "Variant 1",
                "id": "variant_1_id",
                "first_step_ids": ["step_1"],
            }
        ],
        "tags": ["test", "unit-testing"],
        "teams": ["Engineering Team"],
        "steps": [
            {
                "name": "Welcome Email",
                "type": "email",
                "id": "step_1",
                "next_step_ids": [],
                "next_paths": [],
                "channels": ["email"],
                "messages": {
                    "message_1": {
                        "channel": "email",
                        "subject": "Welcome!",
                        "body": "<html><body>Welcome!</body></html>",
                    }
                },
            }
        ],
    }


@pytest.fixture
def sample_canvas_data_summary():
    return {
        "message": "success",
        "data": {
            "name": "Test Canvas",
            "total_stats": {
                "revenue": 1234.56,
                "conversions": 42,
                "conversions_by_entry_time": 38,
                "entries": 150,
            },
            "variant_stats": {
                "variant_1_id": {
                    "name": "Variant 1",
                    "revenue": 823.45,
                    "conversions": 28,
                    "entries": 100,
                }
            },
            "step_stats": {
                "step_1_id": {
                    "name": "Welcome Email",
                    "revenue": 500.25,
                    "conversions": 15,
                    "conversions_by_entry_time": 12,
                    "messages": {
                        "email": [
                            {
                                "sent": 100,
                                "opens": 75,
                                "influenced_opens": 60,
                                "bounces": 5,
                            }
                        ],
                        "android_push": [
                            {
                                "sent": 50,
                                "opens": 30,
                                "influenced_opens": 25,
                                "bounces": 2,
                            }
                        ],
                    },
                }
            },
        },
    }


@pytest.fixture
def sample_canvas_data_series():
    """Sample Canvas data series API response"""
    return {
        "message": "success",
        "data": {
            "name": "Test Canvas Data Series",
            "stats": [
                {
                    "time": "2023-01-01",
                    "total_stats": {},
                    "variant_stats": {
                        "variant-1-id": {
                            "name": "Control",
                            "revenue": 600.30,
                            "conversions": 15,
                            "conversions_by_entry_time": 13,
                            "entries": 60,
                        },
                        "variant-2-id": {
                            "name": "Treatment",
                            "revenue": 400.20,
                            "conversions": 10,
                            "conversions_by_entry_time": 9,
                            "entries": 40,
                        },
                    },
                    "step_stats": {
                        "step-1-id": {
                            "name": "Welcome Email",
                            "revenue": 500.25,
                            "conversions": 12,
                        },
                    },
                }
            ],
        },
    }



from unittest.mock import patch

import pytest

from braze_mcp.models import ContentBlockInfo, ContentBlocksResponse
from braze_mcp.tools.templates import (
    get_content_block_info,
    get_content_blocks,
)


@pytest.fixture
def sample_content_blocks_data():
    """Sample data for content blocks endpoint."""
    return {
        "count": 2,
        "content_blocks": [
            {
                "content_block_id": "cb_12345",
                "name": "Welcome Email Header",
                "content_type": "html",
                "liquid_tag": "{% contentblock 'cb_12345' %}",
                "inclusion_count": 15,
                "created_at": "2023-01-15T10:30:00.000Z",
                "last_edited": "2023-06-20T14:45:00.000Z",
                "tags": ["email", "welcome", "header"],
            },
            {
                "content_block_id": "cb_67890",
                "name": "Footer Disclaimer",
                "content_type": "text",
                "liquid_tag": "{% contentblock 'cb_67890' %}",
                "inclusion_count": 42,
                "created_at": "2023-02-01T08:15:00.000Z",
                "last_edited": "2023-05-10T12:30:00.000Z",
                "tags": ["footer", "legal"],
            },
        ],
    }


@pytest.fixture
def empty_content_blocks_data():
    """Sample data for empty content blocks response."""
    return {"count": 0, "content_blocks": []}


@pytest.fixture
def sample_content_block_info_data():
    """Sample data for content block info endpoint without inclusion data."""
    return {
        "content_block_id": "92ed12a2-efd5-4967-9283-cfb8a5254581",
        "name": "1TestErikaSemtei1",
        "content": '<table border="0" cellpadding="0" cellspacing="0" role="presentation" width="100%">\n\t<tbody>\n\t\t<tr>\n\t\t\t<td align="center">\n\t\t\t<div><img data-bind="attr: {\'title\': $data.name, \'src\': $data.fullUrl}" src="https://cdn.braze.eu/appboy/communication/assets/image_assets/images/5c140b8c3f0da278c03465db/original.png?1544817548" style="width: 100%;" title="hearts_centered.png" /><br />\n\t\t\t </div>\n\n\t\t\t<div style="text-align: center;"><strong><span style="font-size:18px;"><span style="font-family:Arial,Helvetica Neue,Helvetica,sans-serif;">Test Test Test</span></span></strong><br />\n\t\t\t </div>\n\t\t\t</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td align="center"><span style="color:696969;"><span style="font-family:Arial,Helvetica Neue,Helvetica,sans-serif;"><span style="font-size:14px;">I\'m Erika. This is a Test <br />\n\t\t\t<br />\n\t\t</span></span></span></td>\n\t\t</tr>\n\t</tbody>\n</table>\n',
        "description": "",
        "content_type": "html",
        "created_at": "2019-01-17T10:55:57.174+00:00",
        "last_edited": "2019-01-17T10:57:41.528+00:00",
        "tags": [],
        "inclusion_count": 16,
        "message": "success",
    }


@pytest.fixture
def sample_content_block_info_with_inclusion_data():
    """Sample data for content block info endpoint with inclusion data."""
    return {
        "content_block_id": "92ed12a2-efd5-4967-9283-cfb8a5254581",
        "name": "1TestErikaSemtei1",
        "content": '<table border="0" cellpadding="0" cellspacing="0" role="presentation" width="100%">\n\t<tbody>\n\t\t<tr>\n\t\t\t<td align="center">\n\t\t\t<div><img data-bind="attr: {\'title\': $data.name, \'src\': $data.fullUrl}" src="https://cdn.braze.eu/appboy/communication/assets/image_assets/images/5c140b8c3f0da278c03465db/original.png?1544817548" style="width: 100%;" title="hearts_centered.png" /><br />\n\t\t\t </div>\n\n\t\t\t<div style="text-align: center;"><strong><span style="font-size:18px;"><span style="font-family:Arial,Helvetica Neue,Helvetica,sans-serif;">Test Test Test</span></span></strong><br />\n\t\t\t </div>\n\t\t\t</td>\n\t\t</tr>\n\t\t<tr>\n\t\t\t<td align="center"><span style="color:696969;"><span style="font-family:Arial,Helvetica Neue,Helvetica,sans-serif;"><span style="font-size:14px;">I\'m Erika. This is a Test <br />\n\t\t\t<br />\n\t\t</span></span></span></td>\n\t\t</tr>\n\t</tbody>\n</table>\n',
        "description": "",
        "content_type": "html",
        "created_at": "2019-01-17T10:55:57.174+00:00",
        "last_edited": "2019-01-17T10:57:41.528+00:00",
        "tags": [],
        "inclusion_count": 16,
        "message": "success",
        "inclusion_data": [
            {
                "campaign_id": "4bd97844-09e4-4b84-9de3-943cd3fdc420",
                "message_variation_id": "4090cce3-ebd4-40dd-833d-046872660ffa",
            },
            {
                "canvas_step_id": "a6764072-f389-4c67-86fc-6607be1be7dd",
                "message_variation_id": "4222dd15-1cbc-4c9d-be99-d02e8a7850ed",
            },
            {
                "canvas_step_id": "f757bd0f-0deb-4aab-81a5-883b175209d0",
                "message_variation_id": "3631904f-9c2a-4777-940b-118820ab7116",
            },
        ],
    }


@pytest.fixture
def sample_email_templates_data():
    """Sample data for email templates endpoint."""
    return {
        "count": 2,
        "templates": [
            {
                "email_template_id": "template_12345",
                "template_name": "Welcome Email Template",
                "created_at": "2023-01-15T10:30:00.000Z",
                "updated_at": "2023-06-20T14:45:00.000Z",
                "tags": ["welcome", "onboarding", "email"],
            },
            {
                "email_template_id": "template_67890",
                "template_name": "Newsletter Template",
                "created_at": "2023-02-01T08:15:00.000Z",
                "updated_at": "2023-05-10T12:30:00.000Z",
                "tags": ["newsletter", "marketing"],
            },
        ],
    }


@pytest.fixture
def empty_email_templates_data():
    """Sample data for empty email templates response."""
    return {"count": 0, "templates": []}


class TestGetContentBlocks:
    """Test get_content_blocks function"""

    @pytest.mark.asyncio
    async def test_success_response_with_content_blocks(
        self, mock_context, mock_braze_context, sample_content_blocks_data
    ):
        """Test successful content blocks retrieval with data."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=SuccessResponse(data=sample_content_blocks_data, headers={}),
            ) as mock_request,
        ):
            result = await get_content_blocks(mock_context)

            assert isinstance(result, ContentBlocksResponse)
            assert result.count == 2
            assert len(result.content_blocks) == 2

            # Check first content block
            first_block = result.content_blocks[0]
            assert first_block.content_block_id == "cb_12345"
            assert first_block.name == "Welcome Email Header"
            assert first_block.content_type == "html"
            assert first_block.liquid_tag == "{% contentblock 'cb_12345' %}"
            assert first_block.inclusion_count == 15
            assert first_block.created_at == "2023-01-15T10:30:00.000Z"
            assert first_block.last_edited == "2023-06-20T14:45:00.000Z"
            assert first_block.tags == ["email", "welcome", "header"]

            # Check second content block
            second_block = result.content_blocks[1]
            assert second_block.content_block_id == "cb_67890"
            assert second_block.name == "Footer Disclaimer"
            assert second_block.content_type == "text"
            assert second_block.inclusion_count == 42
            assert second_block.tags == ["footer", "legal"]

            # Verify call was made correctly
            call_args = mock_request.call_args
            assert call_args[0][0] == mock_braze_context.http_client
            assert call_args[0][1] == mock_braze_context.base_url
            assert call_args[0][2] == "content_blocks/list"
            params = call_args[0][3]
            assert len(params) == 4

    @pytest.mark.asyncio
    async def test_success_response_with_parameters(
        self, mock_context, mock_braze_context, sample_content_blocks_data
    ):
        """Test content blocks retrieval with all parameters."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=SuccessResponse(data=sample_content_blocks_data, headers={}),
            ) as mock_request,
        ):
            result = await get_content_blocks(
                mock_context,
                modified_after="2023-01-01T00:00:00.000Z",
                modified_before="2023-12-31T23:59:59.999Z",
                limit=50,
                offset=10,
            )

            assert isinstance(result, ContentBlocksResponse)
            assert result.count == 2

            # Verify parameters were passed correctly
            call_args = mock_request.call_args
            params = call_args[0][3]
            assert params["modified_after"] == "2023-01-01T00:00:00.000Z"
            assert params["modified_before"] == "2023-12-31T23:59:59.999Z"
            assert params["limit"] == 50
            assert params["offset"] == 10

    @pytest.mark.asyncio
    async def test_success_response_empty_content_blocks(
        self, mock_context, mock_braze_context, empty_content_blocks_data
    ):
        """Test successful response with no content blocks."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=SuccessResponse(data=empty_content_blocks_data, headers={}),
            ),
        ):
            result = await get_content_blocks(mock_context)

            assert isinstance(result, ContentBlocksResponse)
            assert result.count == 0
            assert len(result.content_blocks) == 0

    @pytest.mark.asyncio
    async def test_api_error_response(self, mock_context, mock_braze_context):
        """Test handling of API error response."""
        from braze_mcp.utils.http import FailureResponse

        error_data = {"error": "Invalid API key"}

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=FailureResponse(data=error_data, error=Exception("API Error")),
            ),
        ):
            result = await get_content_blocks(mock_context)

            # Should return error dict when there's an API error
            assert isinstance(result, dict)
            assert result == error_data

    @pytest.mark.asyncio
    async def test_invalid_response_format(self, mock_context, mock_braze_context):
        """Test handling of invalid response format that can't be parsed by pydantic."""
        from braze_mcp.utils.http import SuccessResponse

        invalid_data = {"invalid_field": "this doesn't match the expected schema"}

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=SuccessResponse(data=invalid_data, headers={}),
            ),
        ):
            result = await get_content_blocks(mock_context)

            # Should return raw dict when pydantic parsing fails
            assert isinstance(result, dict)
            assert result == invalid_data

    @pytest.mark.asyncio
    async def test_content_blocks_unexpected_response_type(self, mock_context, mock_braze_context):
        """Test content blocks handles unexpected response type"""
        from unittest.mock import MagicMock

        # Create a mock object that isn't SuccessResponse or FailureResponse
        unexpected_response = MagicMock()
        unexpected_response.__class__.__name__ = "UnexpectedResponse"

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=unexpected_response,
            ),
            patch("braze_mcp.tools.templates.logger.error") as mock_logger,
        ):
            result = await get_content_blocks(mock_context)

            assert isinstance(result, dict)
            assert "error" in result
            # Check that we get a standardized error response
            assert result["success"] is False
            assert result["error"]["error_type"] == "unexpected_error"
            mock_logger.assert_called_once_with(
                f"Unexpected response type: {type(unexpected_response)}"
            )


class TestGetContentBlockInfo:
    """Test get_content_block_info function"""

    @pytest.mark.asyncio
    async def test_success_response_without_inclusion_data(
        self, mock_context, mock_braze_context, sample_content_block_info_data
    ):
        """Test successful content block info retrieval without inclusion data."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=SuccessResponse(data=sample_content_block_info_data, headers={}),
            ) as mock_request,
        ):
            result = await get_content_block_info(
                mock_context, content_block_id="92ed12a2-efd5-4967-9283-cfb8a5254581"
            )

            assert isinstance(result, ContentBlockInfo)
            assert result.content_block_id == "92ed12a2-efd5-4967-9283-cfb8a5254581"
            assert result.name == "1TestErikaSemtei1"
            assert result.content_type == "html"
            assert result.description == ""
            assert result.tags == []
            assert result.inclusion_count == 16
            assert result.message == "success"
            assert result.inclusion_data is None
            assert result.created_at == "2019-01-17T10:55:57.174+00:00"
            assert result.last_edited == "2019-01-17T10:57:41.528+00:00"
            assert "Test Test Test" in result.content

            call_args = mock_request.call_args
            assert call_args[0][0] == mock_braze_context.http_client
            assert call_args[0][1] == mock_braze_context.base_url
            assert call_args[0][2] == "content_blocks/info"
            params = call_args[0][3]
            assert params["content_block_id"] == "92ed12a2-efd5-4967-9283-cfb8a5254581"
            assert params["include_inclusion_data"] is False

    @pytest.mark.asyncio
    async def test_success_response_with_inclusion_data(
        self, mock_context, mock_braze_context, sample_content_block_info_with_inclusion_data
    ):
        """Test successful content block info retrieval with inclusion data."""
        from braze_mcp.utils.http import SuccessResponse

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=SuccessResponse(
                    data=sample_content_block_info_with_inclusion_data, headers={}
                ),
            ) as mock_request,
        ):
            result = await get_content_block_info(
                mock_context,
                content_block_id="92ed12a2-efd5-4967-9283-cfb8a5254581",
                include_inclusion_data=True,
            )

            assert isinstance(result, ContentBlockInfo)
            assert result.content_block_id == "92ed12a2-efd5-4967-9283-cfb8a5254581"
            assert result.name == "1TestErikaSemtei1"
            assert result.inclusion_count == 16
            assert result.message == "success"

            # Check inclusion data
            assert result.inclusion_data is not None
            assert len(result.inclusion_data) == 3

            # Check first inclusion (campaign)
            first_inclusion = result.inclusion_data[0]
            assert first_inclusion.campaign_id == "4bd97844-09e4-4b84-9de3-943cd3fdc420"
            assert first_inclusion.message_variation_id == "4090cce3-ebd4-40dd-833d-046872660ffa"
            assert first_inclusion.canvas_step_id is None

            # Check second inclusion (canvas step)
            second_inclusion = result.inclusion_data[1]
            assert second_inclusion.canvas_step_id == "a6764072-f389-4c67-86fc-6607be1be7dd"
            assert second_inclusion.message_variation_id == "4222dd15-1cbc-4c9d-be99-d02e8a7850ed"
            assert second_inclusion.campaign_id is None

            call_args = mock_request.call_args
            params = call_args[0][3]
            assert params["content_block_id"] == "92ed12a2-efd5-4967-9283-cfb8a5254581"
            assert params["include_inclusion_data"] is True

    @pytest.mark.asyncio
    async def test_api_error_response(self, mock_context, mock_braze_context):
        """Test handling of API error response."""
        from braze_mcp.utils.http import FailureResponse

        error_data = {"error": "Content Block not found"}

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=FailureResponse(data=error_data, error=Exception("API Error")),
            ),
        ):
            result = await get_content_block_info(mock_context, content_block_id="invalid-id")

            # Should return error dict when there's an API error
            assert isinstance(result, dict)
            assert result == error_data

    @pytest.mark.asyncio
    async def test_invalid_response_format(self, mock_context, mock_braze_context):
        """Test handling of invalid response format that can't be parsed by pydantic."""
        from braze_mcp.utils.http import SuccessResponse

        invalid_data = {"invalid_field": "this doesn't match the expected schema"}

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=SuccessResponse(data=invalid_data, headers={}),
            ),
        ):
            result = await get_content_block_info(mock_context, content_block_id="some-id")

            # Should return raw dict when pydantic parsing fails
            assert isinstance(result, dict)
            assert result == invalid_data

    @pytest.mark.asyncio
    async def test_unexpected_response_type(self, mock_context, mock_braze_context):
        """Test content block info handles unexpected response type"""
        from unittest.mock import MagicMock

        # Create a mock object that isn't SuccessResponse or FailureResponse
        unexpected_response = MagicMock()
        unexpected_response.__class__.__name__ = "UnexpectedResponse"

        with (
            patch(
                "braze_mcp.tools.templates.get_braze_context",
                return_value=mock_braze_context,
            ),
            patch(
                "braze_mcp.tools.templates.make_request",
                return_value=unexpected_response,
            ),
            patch("braze_mcp.tools.templates.logger.error") as mock_logger,
        ):
            result = await get_content_block_info(mock_context, content_block_id="some-id")

            assert isinstance(result, dict)
            assert "error" in result
            # Check that we get a standardized error response
            assert result["success"] is False
            assert result["error"]["error_type"] == "unexpected_error"
            mock_logger.assert_called_once_with(
                f"Unexpected response type: {type(unexpected_response)}"
            )



from pydantic import Field

from .common import BrazeBaseModel
from .message_statistics import MessageStatistics


class SendDataSeriesData(BrazeBaseModel):
    """
    Model representing a single send data series data point.
    """

    time: str = Field(..., description="The date as ISO 8601 date")
    messages: MessageStatistics = Field(..., description="Message statistics by channel")
    conversions_by_send_time: int | None = Field(None, description="Conversions by send time")
    conversions1_by_send_time: int | None = Field(None, description="Conversions1 by send time")
    conversions2_by_send_time: int | None = Field(None, description="Conversions2 by send time")
    conversions3_by_send_time: int | None = Field(None, description="Conversions3 by send time")
    conversions: int = Field(..., description="Total conversions")
    conversions1: int | None = Field(None, description="Conversions1")
    conversions2: int | None = Field(None, description="Conversions2")
    conversions3: int | None = Field(None, description="Conversions3")
    unique_recipients: int = Field(..., description="Number of unique recipients")
    revenue: float | None = Field(None, description="Revenue generated")


class SendDataSeriesResponse(BrazeBaseModel):
    """
    Model representing the response from the sends/data_series endpoint.
    """

    message: str = Field(
        ...,
        description="The status of the export, returns 'success' when completed without errors",
    )
    data: list[SendDataSeriesData] = Field(..., description="List of send data series data points")



import logging
from unittest.mock import MagicMock, patch

from braze_mcp.utils.logging import get_logger


class TestGetLogger:
    """Test the get_logger function"""

    def test_get_logger_basic(self):
        """Test basic logger creation"""
        logger = get_logger("test_logger")

        assert isinstance(logger, logging.Logger)
        assert logger.name == "test_logger"

    def test_get_logger_with_string_level(self):
        """Test logger creation with string log level"""
        with patch("braze_mcp.utils.logging.logging.basicConfig") as mock_basicConfig:
            with patch("braze_mcp.utils.logging.logging.getLogger") as mock_getLogger:
                mock_logger = MagicMock()
                mock_getLogger.return_value = mock_logger

                logger = get_logger("test_logger", "DEBUG")

                mock_basicConfig.assert_called_once_with(
                    level="DEBUG",
                    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
                )
                mock_getLogger.assert_called_once_with("test_logger")
                assert logger == mock_logger

    def test_get_logger_with_int_level(self):
        """Test logger creation with integer log level"""
        with patch("braze_mcp.utils.logging.logging.basicConfig") as mock_basicConfig:
            with patch("braze_mcp.utils.logging.logging.getLogger") as mock_getLogger:
                mock_logger = MagicMock()
                mock_getLogger.return_value = mock_logger

                logger = get_logger("test_logger", logging.ERROR)

                mock_basicConfig.assert_called_once_with(
                    level=logging.ERROR,
                    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
                )
                mock_getLogger.assert_called_once_with("test_logger")
                assert logger == mock_logger

    def test_get_logger_default_level(self):
        """Test logger creation with default INFO level"""
        with patch("braze_mcp.utils.logging.logging.basicConfig") as mock_basicConfig:
            with patch("braze_mcp.utils.logging.logging.getLogger") as mock_getLogger:
                mock_logger = MagicMock()
                mock_getLogger.return_value = mock_logger

                logger = get_logger("test_logger")

                mock_basicConfig.assert_called_once_with(
                    level="INFO",
                    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
                )
                mock_getLogger.assert_called_once_with("test_logger")
                assert logger == mock_logger



"""Shared test fixtures for Braze MCP tools tests."""

from unittest.mock import AsyncMock, MagicMock

import pytest

from braze_mcp.utils.context import BrazeContext


@pytest.fixture
def mock_context():
    """Create a mock MCP context with embedded BrazeContext."""
    mock_ctx = MagicMock()
    mock_ctx.request_context = MagicMock()
    mock_ctx.request_context.lifespan_context = BrazeContext(
        api_key="test_api_key",
        base_url="https://test.braze.com",
        http_client=AsyncMock(),
    )
    return mock_ctx


@pytest.fixture
def mock_braze_context():
    """Create a mock BrazeContext for testing."""
    return BrazeContext(
        api_key="test_api_key",
        base_url="https://test.braze.com",
        http_client=AsyncMock(),
    )



from pydantic import Field

from .common import BrazeBaseModel


class NewUsersDataPoint(BrazeBaseModel):
    """
    Model representing a single data point for new users.
    """

    time: str = Field(..., description="The date as ISO 8601 date")
    new_users: int = Field(..., description="The number of daily new users")


class NewUsersDataSeriesResponse(BrazeBaseModel):
    """
    Model representing the response from the kpi/new_users/data_series endpoint.
    """

    message: str = Field(
        ...,
        description="The status of the export, returns 'success' when completed without errors",
    )
    data: list[NewUsersDataPoint] = Field(..., description="List of new users data points")


class DAUDataPoint(BrazeBaseModel):
    """
    Model representing a single data point for daily active users.
    """

    time: str = Field(..., description="The date as ISO 8601 date")
    dau: int = Field(..., description="The number of daily active users")


class DAUDataSeriesResponse(BrazeBaseModel):
    """
    Model representing the response from the kpi/dau/data_series endpoint.
    """

    message: str = Field(
        ...,
        description="The status of the export, returns 'success' when completed without errors",
    )
    data: list[DAUDataPoint] = Field(..., description="List of daily active users data points")


class MAUDataPoint(BrazeBaseModel):
    """
    Model representing a single data point for monthly active users.
    """

    time: str = Field(..., description="The date as ISO 8601 date")
    mau: int = Field(..., description="The number of monthly active users")


class MAUDataSeriesResponse(BrazeBaseModel):
    """
    Model representing the response from the kpi/mau/data_series endpoint.
    """

    message: str = Field(
        ...,
        description="The status of the export, returns 'success' when completed without errors",
    )
    data: list[MAUDataPoint] = Field(..., description="List of monthly active users data points")


class UninstallsDataPoint(BrazeBaseModel):
    """
    Model representing a single data point for app uninstalls.
    """

    time: str = Field(..., description="The date as ISO 8601 date")
    uninstalls: int = Field(..., description="The number of uninstalls")


class UninstallsDataSeriesResponse(BrazeBaseModel):
    """
    Model representing the response from the kpi/uninstalls/data_series endpoint.
    """

    message: str = Field(
        ...,
        description="The status of the export, returns 'success' when completed without errors",
    )
    data: list[UninstallsDataPoint] = Field(..., description="List of uninstalls data points")



"""Real API validation tests for KPI tools."""

import pytest

from braze_mcp.models.kpi import DAUDataSeriesResponse, NewUsersDataSeriesResponse
from braze_mcp.tools.kpi import get_dau_data_series, get_new_users_data_series


@pytest.mark.real_api
class TestKPIRealAPI:
    """Real API tests for KPI tools."""

    @pytest.mark.asyncio
    async def test_get_dau_data_series_basic(self, real_context):
        """Test get_dau_data_series against real Braze API."""
        result = await get_dau_data_series(real_context, length=7)

        # Validate return type is Pydantic model
        assert isinstance(result, DAUDataSeriesResponse), (
            f"Expected DAUDataSeriesResponse, got {type(result)}"
        )

        # Validate response structure (Pydantic model)
        assert hasattr(result, "data"), "Response should have data attribute"
        assert hasattr(result, "message"), "Response should have message attribute"

        # Validate data types
        assert isinstance(result.data, list), "Data should be a list"
        assert isinstance(result.message, str), "Message should be a string"

    @pytest.mark.asyncio
    async def test_get_new_users_data_series_basic(self, real_context):
        """Test get_new_users_data_series against real Braze API."""
        result = await get_new_users_data_series(real_context, length=7)

        # Validate return type is Pydantic model
        assert isinstance(result, NewUsersDataSeriesResponse), (
            f"Expected NewUsersDataSeriesResponse, got {type(result)}"
        )

        # Validate response structure (Pydantic model)
        assert hasattr(result, "data"), "Response should have data attribute"
        assert hasattr(result, "message"), "Response should have message attribute"

        # Validate data types
        assert isinstance(result.data, list), "Data should be a list"
        assert isinstance(result.message, str), "Message should be a string"
